package com.sillycycle.bagleyd.oct;

/*
 * @(#)OctSolve.java
 *
 * Copyright 2010 - 2023  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.util.Random;
import javax.swing.JFrame;

/**
 * The <code>OctSolve</code> class is the code that does the work
 * of solving the puzzle.
 *
 * @author      David A. Bagley
 * @author      bagleyd AT verizon.net
 * @author      https://www.sillycycle.com/puzzles.html
 */

class OctSolve {
	private static JFrame frame;
	private static OctCanvas oct;
	//private static int counter; // debugging
	private static int returnedFace, returnedPosition;
	private static final boolean DEBUG = false;
	private static Random generator;
	private static int lastCount;

	static final int MAX_CORNERS = 6;

	OctSolve(JFrame frame, OctCanvas oct) {
		OctSolve.frame = frame;
		OctSolve.oct = oct;
		generator = new Random(System.nanoTime());
		//this.counter = 0;
	}

	boolean movePuzzlePiece(int face, int position, int direction, int style,
			boolean control) {
		if (!getFlag()) {
			return false;
		}
		oct.movePuzzleDelay(face, position, direction,
			style, control, OctCanvas.DOUBLE);
		return true;
	}

	static final int[/*OctCanvas.MAX_FACES*/][/*3*/] edgeMateFace =
	{
		{1, 3, 6}, {2, 0, 5}, {3, 1, 4}, {0, 2, 7},
		{5, 7, 2}, {6, 4, 1}, {7, 5, 0}, {4, 6, 3}
	};
	static final int[/*OctCanvas.MAX_FACES*/][/*3*/] edgeMateCorner =
	{
		{3, 1, 6}, {0, 2, 5}, {1, 3, 4}, {2, 0, 7},
		{7, 5, 2}, {4, 6, 1}, {5, 7, 0}, {6, 4, 3}
	};
	static final int[/*MAX_CORNERS*/][/*4*/] facesAtCorner =
	{
		{0, 1, 2, 3},
		{0, 6, 5, 1}, {1, 5, 4, 2}, {2, 4, 7, 3}, {3, 7, 6, 0},
		{6, 7, 4, 5}
	};
	static final int[/*MAX_CORNERS*/][/*4*/] orientAtCorner =
	{
		{0, 0, 0, 0},
		{1, 2, 1, 2}, {1, 2, 1, 2}, {1, 2, 1, 2}, {1, 2, 1, 2},
		{0, 0, 0, 0}
	};
	static final int[/*OctCanvas.MAX_FACES*/][/*3*/] facesToCorner =
	{
		{0, 1, 4}, {0, 2, 1}, {0, 3, 2}, {0, 4, 3},
		{5, 2, 3}, {5, 1, 2}, {5, 4, 1}, {5, 3, 4}
	};
	static final int[/*OctCanvas.MAX_FACES*/][/*3*/][/*3*/] faceCornerToOther =
	{
		{{1, 2, 3}, {6, 5, 1}, {3, 7, 6}},
		{{2, 3, 0}, {5, 4, 2}, {0, 6, 5}},
		{{3, 0, 1}, {4, 7, 3}, {1, 5, 4}},
		{{0, 1, 2}, {7, 6, 0}, {2, 4, 7}},
		{{5, 6, 7}, {2, 1, 5}, {7, 3, 2}},
		{{6, 7, 4}, {1, 0, 6}, {4, 2, 1}},
		{{7, 4, 5}, {0, 3, 7}, {5, 1, 0}},
		{{4, 5, 6}, {3, 2, 4}, {6, 0, 3}}
	};
	static final int[/*3*/][/*3*/] orientCornerToOther =
	{
		{0, 0, 0}, {2, 1, 2}, {1, 2, 1}
	};

	static int indexToCorner(int cornerIndex) {
		switch (cornerIndex) { /* Oct -> 0, 4, 8 */
		case 0:
			return 0;
		case 1:
			return (oct.size - 1) * (oct.size - 1);
		case 2:
			return oct.size * oct.size - 1;
		default:
			System.out.println("corner unknown " + cornerIndex);
		}
		return -1;
	}

	static int indexToCornerMate(int cornerIndex) {
		switch (cornerIndex) { /* Oct -> 0, 8, 4 */
		case 0:
			return 0;
		case 1:
			return oct.size * oct.size - 1;
		case 2:
			return (oct.size - 1) * (oct.size - 1);
		default:
			System.out.println("corner unknown " + cornerIndex);
		}
		return -1;
	}

	static int cornerToIndex(int corner) {
		int temp; /* Oct -> {0, -1, -1, -1, 1, -1, -1, -1, 2} */

		if (corner == 0)
			return 0;
		temp = (oct.size - 1) * (oct.size - 1) - corner;
		if (temp == 0)
			return 1;
		else if (temp < 0) {
			temp = oct.size * oct.size - 1 - corner;
			if (temp == 0)
				return 2;
		}
		return -1;
	}

	static int indexToEdge(int edgeIndex) {
		switch (edgeIndex) { /* Oct -> 1, 3, 6 */
		case 0:
			return (oct.size >> 1) * (oct.size >> 1)
				+ (((oct.size & 1) == 0) ? 1 : 0);
		case 1:
			return (oct.size >> 1) * (oct.size >> 1)
				+ 2 * (oct.size >> 1)
				- (((oct.size & 1) == 0) ? 1 : 0);
		case 2:
			return oct.size * oct.size - oct.size;
		default:
			System.out.println("edge unknown " + edgeIndex);
		}
		return -1;
	}

	static int indexToEdgeMate(int edgeIndex) {
		switch (edgeIndex) { /* Oct -> 3, 1, 6 */
		case 0:
			return (oct.size >> 1) * (oct.size >> 1)
				+ 2 * (oct.size >> 1)
				- (((oct.size & 1) == 0) ? 1 : 0);
		case 1:
			return (oct.size >> 1) * (oct.size >> 1)
				+ (((oct.size & 1) == 0) ? 1 : 0);
		case 2:
			return oct.size * oct.size - oct.size;
		default:
			System.out.println("edge unknown " + edgeIndex);
		}
		return -1;
	}

	static int edgeToIndex(int edge) {
		int temp; /* Oct -> {-1, 0, -1, 1, -1, -1, 2, -1, -1} */

		temp = (oct.size >> 1) * (oct.size >> 1)
			+ 2 * (oct.size >> 1)
			- (((oct.size & 1) == 0) ? 1 : 0)
			- edge;
		if (temp == 0) {
			return 1;
		} else if (temp > 0) {
			temp = (oct.size >> 1) * (oct.size >> 1)
				+ (((oct.size & 1) == 0) ? 1 : 0)
				- edge;
			if (temp == 0) {
				return 0;
			}
		} else if (temp < 0) {
			temp = oct.size * oct.size - oct.size
				- edge;
			if (temp == 0) {
				return 2;
			}
		}
		return -1;
	}

	static int indexToCenter(int centerIndex) {
		switch (centerIndex) { /* Oct -> 2, 5, 7 */
		case 0:
			return 2;
		case 1:
			return (oct.size - 1) * (oct.size - 1) + 1;
		case 2:
			return oct.size * oct.size - 2;
		default:
			System.out.println("center unknown " + centerIndex);
		}
		return -1;
	}

	/* static int centerToIndex(int center) {
		int temp; // Oct-> {-1, -1, 0, -1, -1, 1, -1, 2, -1}

		if (center == 2)
			return 0;
		temp = (oct.size - 1) * (oct.size - 1) + 1 - center;
		if (temp == 0)
			return 2;
		else if (temp < 0) {
			temp = oct.size * oct.size - 2 - center;
			if (temp == 0)
				return 1;
		}
		return -1;
	}*/

	void rotateCornerEdgeCW(int corner) {
		int face = facesAtCorner[corner][3];
		int position = indexToEdge(orientAtCorner[corner][3]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.TL / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateCornerEdgeCCW(int corner) {
		int face = facesAtCorner[corner][0];
		int position = indexToEdgeMate(orientAtCorner[corner][0]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CCW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.BR / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateCornerEdgeHalf(int corner) {
		if (generator.nextInt(2) == 0) {
			rotateCornerEdgeCW(corner);
			rotateCornerEdgeCW(corner);
		} else {
			rotateCornerEdgeCCW(corner);
			rotateCornerEdgeCCW(corner);
		}
	}

	void rotateCornerCenterCW(int corner) {
		int face = facesAtCorner[corner][3];
		int position = indexToCenter(orientAtCorner[corner][3]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.TL / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateCornerCenterCCW(int corner) {
		int face = facesAtCorner[corner][0];
		int position = indexToCenter(orientAtCorner[corner][0]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CCW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.BR / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateCornerCenterHalf(int corner) {
		if (generator.nextInt(2) == 0) {
			rotateCornerCenterCW(corner);
			rotateCornerCenterCW(corner);
		} else {
			rotateCornerCenterCCW(corner);
			rotateCornerCenterCCW(corner);
		}
	}

	void rotatePuzzleCW(int corner) {
		int face = facesAtCorner[corner][3];
		int position = indexToEdge(orientAtCorner[corner][3]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CW, OctInterface.PERIOD4, true);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.TL / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, true);
	}

	void rotatePuzzleCCW(int corner) {
		int face = facesAtCorner[corner][0];
		int position = indexToEdge(orientAtCorner[corner][0]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CCW, OctInterface.PERIOD4, true);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.BR / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, true);
	}

	void rotatePuzzleHalf(int corner) {
		if (generator.nextInt(2) == 0) {
			rotatePuzzleCW(corner);
			rotatePuzzleCW(corner);
		} else {
			rotatePuzzleCCW(corner);
			rotatePuzzleCCW(corner);
		}
	}

	void rotateTrivialCornerCW(int corner) {
		int face = facesAtCorner[corner][3];
		int position = indexToCorner(orientAtCorner[corner][3]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.TL / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateTrivialCornerCCW(int corner) {
		int face = facesAtCorner[corner][0];
		int position = indexToCorner(orientAtCorner[corner][0]);

		if (corner == 0 || corner == 5)
			movePuzzlePiece(face, position, OctCanvas.CCW, OctInterface.PERIOD4, false);
		else
			movePuzzlePiece(face, position,
				((OctCanvas.BR / 2 + corner - 1) % 4 * 2 + OctCanvas.TR), OctInterface.PERIOD4, false);
	}

	void rotateTrivialCornerHalf(int corner) {
		if (generator.nextInt(2) == 0) {
			rotateTrivialCornerCW(corner);
			rotateTrivialCornerCW(corner);
		} else {
			rotateTrivialCornerCCW(corner);
			rotateTrivialCornerCCW(corner);
		}
	}

	/* This is impossible to do with Drag and Drop as implemented.
	 * Use the keypad.  Turn the trivial corners so that the
	 * colors match the adjacent pieces all around. */
	void orientTrivialCorners(int corner) {
		int face = facesAtCorner[corner][0];
		int orient = orientAtCorner[corner][0];
		int cornerPosition = indexToCorner(orient);
		int centerPosition = indexToCenter(orient);
		int cornerColor = oct.facetLoc[face][cornerPosition].face;
		int centerColor = oct.facetLoc[face][centerPosition].face;

		if (cornerColor == centerColor) {
			return;
		}
		face = facesAtCorner[corner][1];
		orient = orientAtCorner[corner][1];
		centerPosition = indexToCenter(orient);
		centerColor = oct.facetLoc[face][centerPosition].face;
		if (cornerColor == centerColor) {
			rotateTrivialCornerCW(corner);
			return;
		}
		face = facesAtCorner[corner][2];
		orient = orientAtCorner[corner][2];
		centerPosition = indexToCenter(orient);
		centerColor = oct.facetLoc[face][centerPosition].face;
		if (cornerColor == centerColor) {
			rotateTrivialCornerHalf(corner);
			return;
		}
		face = facesAtCorner[corner][3];
		orient = orientAtCorner[corner][3];
		centerPosition = indexToCenter(orient);
		centerColor = oct.facetLoc[face][centerPosition].face;
		if (cornerColor == centerColor) {
			rotateTrivialCornerCCW(corner);
			return;
		}
		System.out.println("no match " + corner
			+ ", cornerColor" + cornerColor
			+ ", centerColor" + centerColor);
	}

	static int findFaceColor(int face) {
		int[] colors = new int[8];
		int point, color;

		for (color = 0; color < OctCanvas.MAX_FACES; color++)
			colors[color] = 0;
		for (point = 0; point < 3; point++) {
			int side;
			int corner = facesToCorner[face][point];

			for (side = 0; side < 4; side++) {
				int otherFace = facesAtCorner[corner][side];
				int position = indexToCenter(
					orientAtCorner[corner][side]);

				color = oct.facetLoc[otherFace][position].face;
				colors[color]++;
				if (colors[color] == 3) {
					if (DEBUG)
						System.out.println("Color for face " + face + " is " + color);
					return color;
				}
			}
		}
		System.out.println("No color found for " + face);
		return OctCanvas.MAX_FACES;
	}

	void faceCorner(int face, int faceColor, int point) {
		int side;
		int corner = facesToCorner[face][point];
		int otherFace = face;
		int otherOrient = point;

		for (side = 0; side < 4; side++) {
			int position = indexToCenter(otherOrient);

			if (faceColor == oct.facetLoc[otherFace][position].face) {
				switch(side) {
				case 1:
					rotateCornerCenterCCW(corner);
					break;
				case 2:
					rotateCornerCenterHalf(corner);
					break;
				case 3:
					rotateCornerCenterCW(corner);
					break;
				default:
				}
				return;
			} else if (side < 3) {
				otherFace = faceCornerToOther[face][point][side];
				otherOrient = orientCornerToOther[point][side];
			}
		}
	}

	void faceCorners(int face, int faceColor) {
		int point;

		for (point = 0; point < 3; point++) {
			faceCorner(face, faceColor, point);
		}
	}

	void faceTrivialCorners() {
		int point;

		for (point = 0; point < 3; point++) {
			orientTrivialCorners(facesToCorner[0][point]);
		}
	}

	void allTrivialCorners() {
		int corner;

		for (corner = 0; corner < MAX_CORNERS; corner++) {
			orientTrivialCorners(corner);
		}
	}

	static boolean checkPiece(int color, int face, int position) {
		int newFace, newPosition, positionIndex;

		positionIndex = edgeToIndex(position);
		if (positionIndex == -1) {
			positionIndex = 0;
			System.out.println("position " + position + " incorrect");
		}
		newFace = edgeMateFace[face][positionIndex];
		newPosition = indexToEdgeMate(positionIndex);
		return (oct.facetLoc[newFace][newPosition].face == color);
	}

	static void findPiece(int color0, int color1) {
		// returns returnedFace, returnedPosition;
		int positionIndex;

		for (returnedFace = 0; returnedFace < OctCanvas.MAX_FACES; returnedFace++) {
			for (positionIndex = 0; positionIndex < 3; positionIndex++) {
				returnedPosition = indexToEdge(positionIndex);
				if (oct.facetLoc[returnedFace][returnedPosition].face == color0 &&
						checkPiece(color1, returnedFace, returnedPosition)) {
					return;
				}
			}
		}
		System.out.println("Piece " + color0 + " " + color1 + " not found!");
	}

	static boolean onFace(int target, int face, int position) {
		int side;

		if (face == target)
			return true;
		for (side = 0; side < 3; side++) {
			if (face == edgeMateFace[target][side] &&
					position == indexToEdgeMate(side))
				return true;
		}
		return false;
	}

	static boolean bottomRing(int face, int position) {
		return (face > 3 && position != indexToEdge(2));
	}

	static boolean rightRing(int face, int position) {
		return (((face == 1 || face == 2 || face == 4 || face == 5) &&
				 position == indexToEdge(2)) ||
			(face == 1 && position == indexToEdge(0)) ||
			(face == 2 && position == indexToEdge(1)) ||
			(face == 4 && position == indexToEdge(0)) ||
			(face == 5 && position == indexToEdge(1)));
	}

	static boolean leftRing(int face, int position) {
		return (((face == 2 || face == 3 || face == 4 || face == 7) &&
				 position == indexToEdge(2)) ||
			(face == 2 && position == indexToEdge(0)) ||
			(face == 3 && position == indexToEdge(1)) ||
			(face == 4 && position == indexToEdge(1)) ||
			(face == 7 && position == indexToEdge(0)));
	}

	void face0Edge6(int matchFace, int faceColor) {
		int otherColor, face, position, matchPosition;

		matchPosition = indexToEdge(2);
		/* need other color */
		face = edgeMateFace[matchFace][2];
		position = matchPosition - oct.size + 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (onFace(0, face, position)) { /* 1/4 of possible */
			switch (face) {
			case 0:
				if (position == indexToEdge(0)) {
					rotateCornerEdgeCW(4);
					rotateCornerEdgeCCW(0);
					rotateCornerEdgeCCW(4);
					rotateCornerEdgeCW(0);
				} else if (position == indexToEdge(1)) {
					rotateCornerEdgeCCW(1);
					rotateCornerEdgeCW(0);
					rotateCornerEdgeCW(1);
					rotateCornerEdgeCCW(0);
				}
				return;
			case 1:
				rotateCornerEdgeCW(1);
				rotateCornerEdgeCCW(4);
				rotateCornerEdgeCCW(1);
				rotateCornerEdgeCW(4);
				return;
			case 3:
				rotateCornerEdgeCCW(4);
				rotateCornerEdgeCW(1);
				rotateCornerEdgeCW(4);
				rotateCornerEdgeCCW(1);
				return;
			default: /* correct but flip */
				if (generator.nextInt(2) == 0) {
					rotateCornerEdgeCW(1);
					rotateCornerEdgeCW(5);
					rotateCornerEdgeCCW(1);
					rotateCornerEdgeCW(4);
					rotateCornerEdgeCCW(1);
					rotateCornerEdgeCCW(4);
					rotateCornerEdgeCW(1);
				} else {
					rotateCornerEdgeCCW(4);
					rotateCornerEdgeCCW(5);
					rotateCornerEdgeCW(4);
					rotateCornerEdgeCCW(1);
					rotateCornerEdgeCW(4);
					rotateCornerEdgeCW(1);
					rotateCornerEdgeCCW(4);
				}
				return;
			}
		}
		if (leftRing(face, position)) {
			if ((face == 3 || face == 7) && position == indexToEdge(2))
				rotateCornerEdgeCCW(3);
			else if ((face == 3 && position == indexToEdge(1)) ||
					(face == 2 && position == indexToEdge(0)))
				rotateCornerEdgeHalf(3);
			else if ((face == 2 || face == 4) && position == indexToEdge(2))
				rotateCornerEdgeCW(3);
		} else if (rightRing(face, position)) {
			if ((face == 1 || face == 5) && position == indexToEdge(2))
				rotateCornerEdgeCW(2);
			else if ((face == 1 && position == indexToEdge(0)) ||
					(face == 2 && position == indexToEdge(1)))
				rotateCornerEdgeHalf(2);
			else if ((face == 2 || face == 4) && position == indexToEdge(2))
				rotateCornerEdgeCCW(2);
		}
		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		face = returnedFace;
		position = returnedPosition;
		if (!bottomRing(face, position)) {
			System.out.println("not in bottom ring is not possible");
			return;
		}
		switch (face) {
		case 4:
			rotateCornerEdgeHalf(5);
			break;
		case 5:
			rotateCornerEdgeCW(5);
			break;
		case 7:
			rotateCornerEdgeCCW(5);
			break;
		default:
			break;
		}
		if (position == indexToEdge(1)) {
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCCW(4);
		} else {
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(1);
		}
	}

	void face0Edge1(int matchFace, int faceColor) {
		int otherColor, face, position;
		boolean swap = false;

		/* need other color */
		face = edgeMateFace[matchFace][0];
		position = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (onFace(0, face, position)) { /* 1/4 of possible */
			if (face == 0 && position == indexToEdge(0)) {
				return;
			}
			if (face == 1) {
				rotateCornerEdgeCW(0);
				rotateCornerEdgeCW(2);
				rotateCornerEdgeCCW(0);
			} else {
				rotateCornerEdgeHalf(0);
				rotateCornerEdgeCW(2);
				rotateCornerEdgeHalf(0);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (leftRing(face, position)) {
			if ((face == 3 || face == 7) && position == indexToEdge(2))
				rotateCornerEdgeHalf(3);
			else if ((face == 2 && position == indexToEdge(0)) || face == 3)
				rotateCornerEdgeCW(3);
			else if ((face == 4 && position == indexToEdge(1)) || face == 7)
				rotateCornerEdgeCCW(3);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		} else if (bottomRing(face, position)) {
			if ((face == 7 && position == indexToEdge(1)) ||
					(face == 6 && position == indexToEdge(0)))
				rotateCornerEdgeHalf(5);
			else if ((face == 4 && position == indexToEdge(1)) ||
					(face == 7 && position == indexToEdge(0)))
				rotateCornerEdgeCW(5);
			else if ((face == 6 && position == indexToEdge(1)) ||
					(face == 5 && position == indexToEdge(0)))
				rotateCornerEdgeCCW(5);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!rightRing(face, position)) {
			System.out.println("not in rightRing is not possible");
			return;
		}
		switch (face) {
		case 2:
			rotateCornerEdgeCW(2);
			swap = true;
			break;
		case 4:
			rotateCornerEdgeHalf(2);
			break;
		case 5:
			rotateCornerEdgeCCW(2);
			swap = true;
			break;
		default:
			break;
		}
		if ((position == indexToEdge(2) && !swap) || (position != indexToEdge(2) && swap)) {
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCW(0);
		} else {
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCW(0);
			rotateCornerEdgeCCW(1);
		}
	}

	void face0Edge3(int matchFace, int faceColor) {
		int otherColor, face, position;
		boolean swap = false;

		/* need other color */
		face = edgeMateFace[matchFace][1];
		position = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (onFace(0, face, position)) { /* 1/4 of possible */
			if (face == 0 && position == indexToEdge(1)) {
				return;
			}
			/* face == 3 */
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(0);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (rightRing(face, position)) {
			if ((face == 1 || face == 5) && position == indexToEdge(2))
				rotateCornerEdgeHalf(2);
			else if ((face == 2 && position == indexToEdge(1)) || face == 1)
				rotateCornerEdgeCCW(2);
			else if ((face == 4 && position == indexToEdge(0)) || face == 5)
				rotateCornerEdgeCW(2);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		} else if (bottomRing(face, position)) {
			if ((face == 5 && position == indexToEdge(0)) ||
					(face == 6 && position == indexToEdge(1)))
				rotateCornerEdgeHalf(5);
			else if ((face == 4 && position == indexToEdge(0)) ||
					(face == 5 && position == indexToEdge(1))) {
				rotateCornerEdgeCCW(5);
			} else if ((face == 6 && position == indexToEdge(0)) ||
					(face == 7 && position == indexToEdge(1))) {
				rotateCornerEdgeCW(5);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!leftRing(face, position)) {
			System.out.println("not in leftRing is not possible");
			return;
		}
		switch (face) {
		case 2:
			rotateCornerEdgeCCW(3);
			swap = true;
			break;
		case 4:
			rotateCornerEdgeHalf(3);
			break;
		case 7:
			rotateCornerEdgeCW(3);
			swap = true;
			break;
		default:
			break;
		}
		if ((position == indexToEdge(2) && !swap) || (position != indexToEdge(2) && swap)) {
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(0);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(0);
		} else {
			rotateCornerEdgeCW(0);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCW(4);
		}
	}

	void face1Edge6(int matchFace, int faceColor) {
		int otherColor, face, position, matchPosition;

		matchPosition = indexToEdge(2);
		/* need other color */
		face = edgeMateFace[matchFace][2];
		position = matchPosition - oct.size + 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (onFace(1, face, position)) {
			if (face == 1 && position == indexToEdge(2)) {
				return;
			}
			if (face == 5 && position == indexToEdge(2)) {
				rotateCornerEdgeCW(2);
				rotateCornerEdgeCW(5);
				rotateCornerEdgeCCW(2);
			} else {
				rotateCornerEdgeHalf(2);
				rotateCornerEdgeCW(5);
				rotateCornerEdgeHalf(2);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		} else if (leftRing(face, position)) {
			if ((face == 3 || face == 7) && position == indexToEdge(2)) {
				rotateCornerEdgeCCW(3);
			} else if ((face == 2 || face == 4) && position == indexToEdge(2)) {
				rotateCornerEdgeCW(3);
			} else if ((face == 3 && position == indexToEdge(1)) ||
					(face == 2 && position == indexToEdge(0))) {
				rotateCornerEdgeHalf(3);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!bottomRing(face, position)) {
			System.out.println("not in bottomRing is not possible");
			return;
		}
		switch (face) {
		case 4:
			rotateCornerEdgeCW(5);
			break;
		case 6:
			rotateCornerEdgeCCW(5);
			break;
		case 7:
			rotateCornerEdgeHalf(5);
			break;
		default:
			break;
		}
		if (position == indexToEdge(0)) {
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCW(2);
		} else {
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(1);
		}
	}

	void face1Edge1(int matchFace, int faceColor) {
		int otherColor, face, position;
		boolean swap = false;

		/* need other color */
		face = edgeMateFace[matchFace][0];
		position = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (onFace(1, face, position)) {
			if (face == 1 && position == indexToEdge(0)) {
				return;
			}
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(2);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		} else if (bottomRing(face, position)) {
			if ((face == 4 && position == indexToEdge(0)) ||
					(face == 5 && position == indexToEdge(1))) {
				rotateCornerEdgeCCW(5);
			} else if ((face == 5 && position == indexToEdge(0)) ||
					(face == 6 && position == indexToEdge(1))) {
				rotateCornerEdgeHalf(5);
			} else if ((face == 6 && position == indexToEdge(0)) ||
					(face == 7 && position == indexToEdge(1))) {
				rotateCornerEdgeCW(5);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!leftRing(face, position)) {
			System.out.println("not in leftRing is not possible");
			return;
		}
		switch (face) {
		case 3:
			rotateCornerEdgeCW(3);
			swap = true;
			break;
		case 4:
			rotateCornerEdgeCCW(3);
			swap = true;
			break;
		case 7:
			rotateCornerEdgeHalf(3);
			break;
		default:
			break;
		}
		if ((position == indexToEdge(2) && !swap) || (position != indexToEdge(2) && swap)) {
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(0);
		} else {
			rotateCornerEdgeCCW(0);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(0);
			rotateCornerEdgeCCW(2);
		}
	}

	void face3Edge6(int matchFace, int faceColor) {
		int otherColor, face, position, matchPosition;

		matchPosition = indexToEdge(2);
		/* need other color */
		face = edgeMateFace[matchFace][2];
		position = matchPosition - oct.size + 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (leftRing(face, position)) {
			if (face == 3 && position == indexToEdge(2)) {
				return;
			}
			if (face == 7 && position == indexToEdge(2)) {
				rotateCornerEdgeCCW(3);
				rotateCornerEdgeCCW(5);
				rotateCornerEdgeCW(3);
			} else if ((face == 3 && position == indexToEdge(1)) ||
					(face == 2 && position == indexToEdge(0))) {
				rotateCornerEdgeHalf(3);
				rotateCornerEdgeCCW(5);
				rotateCornerEdgeHalf(3);
			} else if ((face == 2 || face == 4) && position == indexToEdge(2)) {
				rotateCornerEdgeCW(3);
				rotateCornerEdgeCCW(5);
				rotateCornerEdgeCCW(3);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!bottomRing(face, position)) {
			System.out.println("not in bottomRing is not possible");
			return;
		}
		switch (face) {
		case 4:
			rotateCornerEdgeCCW(5);
			break;
		case 5:
			rotateCornerEdgeHalf(5);
			break;
		case 6:
			rotateCornerEdgeCW(5);
			break;
		default:
			break;
		}
		if (position == indexToEdge(0)) {
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(4);
		} else {
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(3);
		}
	}

	void face3Edge3(int matchFace, int faceColor) {
		int otherColor, face, position;

		/* need other color */
		face = edgeMateFace[matchFace][1];
		position = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (leftRing(face, position)) {
			if (face == 3 && position == indexToEdge(1)) {
				return;
			}
			if (face == 2 && position == indexToEdge(0)) {
				rotateCornerEdgeHalf(3);
				rotateCornerEdgeCW(5);
				rotateCornerEdgeHalf(3);
			} else if (position == indexToEdge(2)) {
				rotateCornerEdgeCW(3);
				rotateCornerEdgeCW(5);
				rotateCornerEdgeCCW(3);
			} else {
				rotateCornerEdgeCW(5);
			}
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!bottomRing(face, position)) {
			System.out.println("not in bottomRing is not possible");
			return;
		}
		rotateCornerEdgeCW(3);
		switch (face) {
		case 5:
			rotateCornerEdgeCCW(5);
			break;
		case 6:
			rotateCornerEdgeHalf(5);
			break;
		case 7:
			rotateCornerEdgeCW(5);
			break;
		default:
			break;
		}
		if (position == indexToEdge(0)) {
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(3);
		} else {
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(2);
		}
		rotateCornerEdgeCCW(3);
	}

	void face2Edge6(int matchFace, int faceColor) {
		int otherColor, face, position, matchPosition;

		matchPosition = indexToEdge(2);
		/* need other color */
		face = edgeMateFace[matchFace][2];
		position = matchPosition - oct.size + 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (leftRing(face, position)) {
			if (face == 2 && position == indexToEdge(2)) {
				return;
			}
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(3);
			findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
			face = returnedFace;
			position = returnedPosition;
		}
		if (!bottomRing(face, position)) {
			System.out.println("not in bottomRing is not possible");
			return;
		}
		switch (face) {
		case 5:
			rotateCornerEdgeCCW(5);
			break;
		case 6:
			rotateCornerEdgeHalf(5);
			break;
		case 7:
			rotateCornerEdgeCW(5);
			break;
		default:
			break;
		}
		if (position == indexToEdge(0)) {
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(3);
		} else {
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(2);
		}
	}

	void face4Edge1(int matchFace, int faceColor) {
		int otherColor, face, position;

		/* need other color */
		face = edgeMateFace[matchFace][0];
		position = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face][position].face;

		findPiece(faceColor, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face = returnedFace;
		position = returnedPosition;
		if (!bottomRing(face, position)) {
			System.out.println("not in bottomRing is not possible");
			return;
		}
		if (face == 4 && position == indexToEdge(0)) {
			return;
		}
		/* try rotating puzzle in case of solved pieces */
		if ((oct.facetLoc[4][indexToEdge(1)].face ==
				oct.facetLoc[4][2].face) &&
				(oct.facetLoc[7][indexToEdge(0)].face ==
				oct.facetLoc[7][2].face)) {
			rotatePuzzleCW(5);
			return;
		} else if ((oct.facetLoc[6][indexToEdge(1)].face ==
				oct.facetLoc[6][2].face) &&
				(oct.facetLoc[5][indexToEdge(0)].face ==
				oct.facetLoc[5][2].face)) {
			rotatePuzzleCCW(5);
			return;
		} else if ((oct.facetLoc[7][indexToEdge(1)].face ==
				oct.facetLoc[7][2].face) &&
				(oct.facetLoc[6][indexToEdge(0)].face ==
				oct.facetLoc[6][2].face)) {
			rotatePuzzleHalf(5);
			return;
		}
		if (face == 4 && position == indexToEdge(1)) {
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeHalf(2);
			rotateCornerEdgeCCW(1);
		} else if (face == 5 && position == indexToEdge(0)) {
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(1);
		} else if (face == 5 && position == indexToEdge(1)) {
			rotateCornerEdgeCW(1);
			rotateCornerEdgeHalf(2);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(1);
		} else if (face == 6 && position == indexToEdge(0)) {
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeHalf(3);
			rotateCornerEdgeCW(4);
		} else if (face == 6 && position == indexToEdge(1)) {
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeHalf(2);
			rotateCornerEdgeCW(3);
		} else if (face == 7 && position == indexToEdge(0)) {
			rotateCornerEdgeCW(1);
			/*rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(2);*/
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(1);
		} else if (face == 7 && position == indexToEdge(1)) {
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(2);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(2);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
		}
	}

	void lastEdges(int matchFace0, int faceColor0,
			int matchFace1, int faceColor1) {
		int otherColor, face0, position0, face1, position1;

		/* need other color */
		face0 = edgeMateFace[matchFace0][1];
		position0 = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face0][position0].face;

		findPiece(faceColor0, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face0 = returnedFace;
		position0 = returnedPosition;

		/* need other color */
		face1 = edgeMateFace[matchFace1][0];
		position1 = 2; /* this is a solved piece */
		otherColor = oct.facetLoc[face1][position1].face;

		findPiece(faceColor1, otherColor); // returns returnedFace, returnedPosition
		/* face and position variables are reused here */
		face1 = returnedFace;
		position1 = returnedPosition;

		/* 12 possibilities */
		if (face0 == 4 && position0 == indexToEdge(1)
				&& face1 == 5 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("done");
			return;
		}
		if (face0 == 4 && position0 == indexToEdge(1) &&
				face1 == 6 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("4363!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(4);
		} else if (face0 == 5 && position0 == indexToEdge(0) &&
				face1 == 6 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("5161!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 5 && position0 == indexToEdge(0) &&
				face1 == 7 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("5173!");
			rotateCornerEdgeCCW(1);
			/*rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(4);*/
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 6 && position0 == indexToEdge(0) &&
				face1 == 4 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("6143!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 6 && position0 == indexToEdge(0) &&
				face1 == 7 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("6171!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeHalf(4);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 6 && position0 == indexToEdge(1) &&
				face1 == 6 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("6361!");
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeHalf(5);
			rotateCornerEdgeCW(1);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
		} else if (face0 == 6 && position0 == indexToEdge(1) &&
				face1 == 7 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("6373!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeHalf(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 7 && position0 == indexToEdge(0) &&
				face1 == 5 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("7151!");
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(3);
		} else if (face0 == 7 && position0 == indexToEdge(0) &&
				face1 == 6 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("7163!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeHalf(4);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else if (face0 == 7 && position0 == indexToEdge(1) &&
				face1 == 4 && position1 == indexToEdge(1)) {
			if (DEBUG)
				System.out.println("7343!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(5);
			rotateCornerEdgeCW(1);
		} else if (face0 == 7 && position0 == indexToEdge(1) &&
				face1 == 7 && position1 == indexToEdge(0)) {
			if (DEBUG)
				System.out.println("7371!");
			rotateCornerEdgeCCW(1);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCCW(4);
			rotateCornerEdgeHalf(5);
			rotateCornerEdgeCCW(3);
			rotateCornerEdgeCCW(5);
			rotateCornerEdgeCW(3);
			rotateCornerEdgeCW(4);
			rotateCornerEdgeCW(1);
		} else {
			if (lastCount == 0) {
				lastCount++;
				if (generator.nextInt(2) == 0)
					rotateCornerEdgeCW(5);
				else
					rotateCornerEdgeCCW(5);
				if (DEBUG)
					System.out.println("try last part again");
				solveLastCorner();
			} else
				System.out.println("13th case, not possible "
					+ face0 + " " + position0 + " "
					+ face1 + " " + position1);
		}
	}

	void solveTopFace() {
		/* Pick a face (0) and determine which color is present at each
		 * corner. */
		int topFace = 0;
		int faceColor = findFaceColor(topFace);

		faceCorners(topFace, faceColor);
		faceTrivialCorners();
		face0Edge6(topFace, faceColor);
		face0Edge1(topFace, faceColor);
		face0Edge3(topFace, faceColor);
	}

	void solveSecondFace() {
		int rightFace = 1;
		int faceColor = findFaceColor(rightFace);

		faceCorner(rightFace, faceColor, 1);
		orientTrivialCorners(facesToCorner[1][1]);
		face1Edge6(rightFace, faceColor);
		face1Edge1(rightFace, faceColor);
	}

	void solveThirdCorner() {
		int leftFace = 3, bottomFace = 2;
		int faceColor = findFaceColor(leftFace);

		faceCorner(leftFace, faceColor, 2);
		orientTrivialCorners(facesToCorner[3][2]);
		face3Edge6(leftFace, faceColor);
		face3Edge3(leftFace, faceColor);
		faceColor = findFaceColor(bottomFace);
		face2Edge6(bottomFace, faceColor);
	}

	void solveLastCorner() {
		int bottomFace = 4, rightFace = 5;
		int faceColor0 = findFaceColor(bottomFace), faceColor1;

		faceCorner(bottomFace, faceColor0, 0);
		orientTrivialCorners(facesToCorner[5][0]);
		/* Now for the hard part... last 4 */
		face4Edge1(bottomFace, faceColor0);
		/* Also relatively hard, redo in case rotated */
		faceColor0 = findFaceColor(bottomFace);
		faceColor1 = findFaceColor(rightFace);
		lastEdges(bottomFace, faceColor0, rightFace, faceColor1);
	}

	/* Size 4 oct */
	/* switch 2 and 3 so can have odd and even have a parity meaning for edgesLR */
	void printFacets() {
		int face, position, square;

		for (face = 0; face < OctCanvas.MAX_FACES; face++) {
			square = 1;
			for (position = 0; position < oct.sizeSize; position++) {
				System.out.println(oct.facetLoc[face][position].face
					+ " " + oct.facetLoc[face][position].rotation);
				if (position == square * square - 1) {
					System.out.println();
					++square;
				}
			}
			System.out.println();
		}
		System.out.println();
	}

	static int edgeLRMateFace[/*OctCanvas.MAX_FACES*/][/*6*/] =
	{
		{1, 3, 3, 1, 6, 6},
		{2, 0, 0, 2, 5, 5},
		{3, 1, 1, 3, 4, 4},
		{0, 2, 2, 0, 7, 7},
		{5, 7, 7, 5, 2, 2},
		{6, 4, 4, 6, 1, 1},
		{7, 5, 5, 7, 0, 0},
		{0, 6, 6, 0, 3, 3}
	};

	int edgeLRMatePositionIndex[/*6*/] =
	{1, 0, 3, 2, 5, 4};

	int indexToEdgeLR(int edgeIndex) {
		switch (edgeIndex) { /* Size 4 Oct -> 1, 3, 4, 8, 11, 13 */
		case 0:
			return 1;
		case 1:
			return 3;
		case 3:
			return (oct.size - 2) * (oct.size - 2);
		case 2:
			return (oct.size - 1) * (oct.size - 1) - 1;
		case 4:
			return (oct.size - 1) * (oct.size - 1) + 2;
		case 5:
			return oct.size * oct.size - 3;
		default:
			System.out.println("edge unknown " + edgeIndex);
		}
		return -1;
	}

	int edgeLRToIndex(int edge) {
		int temp; /* Size 4 Oct -> {-1, 0, -1, 1, 3, -1, -1, -1, 2, -1, -1, 4, -1, 5, -1, -1} */

		if (edge == 1)
			return 0;
		else if (edge == 3)
			return 1;
		temp = (oct.size - 1) * (oct.size - 1) - 1 - edge;
		if (temp == 0)
			return 2;
		else if (temp > 0) {
			temp = (oct.size - 2) * (oct.size - 2) - edge;
			if (temp == 0)
				return 3;
		} else if (temp < 0) {
			if (temp == -3)
				return 4;
			temp = oct.size * oct.size - 3 - edge;
			if (temp == 0)
				return 5;
		}
		return -1;
	}

	boolean checkPieceLR(int color, int face, int position) {
		int newFace, newPosition, positionIndex, newPositionIndex;

		positionIndex = edgeLRToIndex(position);
		if (positionIndex == -1) {
			positionIndex = 0;
			System.out.println("position " + position + " incorrect");
		}
		newFace = edgeLRMateFace[face][positionIndex];
		newPositionIndex = edgeLRMatePositionIndex[positionIndex];
		newPosition = indexToEdgeLR(newPositionIndex);
		return (oct.facetLoc[newFace][newPosition].face == color);
	}

	/* color0 is even position */
	void findPieceLR(int color0, int color1) {
		// returns returnedFace, returnedPosition;
		int faceSkip, positionIndex, startPositionIndex;

		startPositionIndex = edgeLRToIndex(returnedPosition);
		/* Check starting face first */
		for (positionIndex = 0; positionIndex < 6; positionIndex++) {
			returnedPosition = indexToEdgeLR(positionIndex);
			if (((startPositionIndex + positionIndex) % 2) == 0) {
				if (oct.facetLoc[returnedFace][returnedPosition].face == color0 &&
						checkPieceLR(color1, returnedFace, returnedPosition)) {
					if (DEBUG)
						System.out.println("LRe " + returnedFace + " " + returnedPosition);
					return;
				}
			} else {
				if (oct.facetLoc[returnedFace][returnedPosition].face == color1 &&
						checkPieceLR(color0, returnedFace, returnedPosition)) {
					if (DEBUG)
						System.out.println("LRo " + returnedFace + " " + returnedPosition);
					return;
				}
			}
		}
		faceSkip = returnedFace;
		for (returnedFace = 0; returnedFace < OctCanvas.MAX_FACES; returnedFace++) {
			if (returnedFace == faceSkip)
				continue;
			for (positionIndex = 0; positionIndex < 6; positionIndex++) {
				returnedPosition = indexToEdgeLR(positionIndex);
				if (((startPositionIndex + positionIndex) % 2) == 0) {
					if (oct.facetLoc[returnedFace][returnedPosition].face == color0 &&
							checkPieceLR(color1, returnedFace, returnedPosition)) {
						if (DEBUG)
							System.out.println("LR e " + returnedFace + " " + returnedPosition);
						return;
					}
				} else {
					if (oct.facetLoc[returnedFace][returnedPosition].face == color1 &&
							checkPieceLR(color0, returnedFace, returnedPosition)) {
						if (DEBUG)
							System.out.println("LR o " + returnedFace + " " + returnedPosition);
						return;
					}
				}
			}
		}
		System.out.println("PieceLR " + color0 + " "
			+ color1 + " not found!");
		printFacets();
	}

	void solve0LR0() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[1][0].face;
		int startFace = 0, startPositionIndex = 0;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR0 color0=" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
				return;
			case 1:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(2, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve0LR1() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[3][0].face;
		int startFace = 0, startPositionIndex = 1;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR1 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
				System.out.println("cannot happen");
				return;
			case 1:
				return;
			case 2:
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(2, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve0LR2() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[3][0].face;
		int startFace = 0, startPositionIndex = 2;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR2 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
				System.out.println("cannot happen");
				return;
			case 2:
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve0LR3() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[1][0].face;
		int startFace = 0, startPositionIndex = 3;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR3 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
				System.out.println("cannot happen");
				return;
			case 3:
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(2, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve2LR0() {
		int positionIndex;
		int color0 = oct.facetLoc[2][0].face;
		int color1 = oct.facetLoc[3][0].face;
		int startFace = 2, startPositionIndex = 0;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 2LR0 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				return;
			case 1:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve2LR1() {
		int positionIndex;
		int color0 = oct.facetLoc[2][0].face;
		int color1 = oct.facetLoc[1][0].face;
		int startFace = 2, startPositionIndex = 1;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 2LR1 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
				System.out.println("cannot happen");
				return;
			case 1:
				return;
			case 2:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve2LR2() {
		int positionIndex;
		int color0 = oct.facetLoc[2][0].face;
		int color1 = oct.facetLoc[1][0].face;
		int startFace = 2, startPositionIndex = 2;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 2LR2 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(1, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
				System.out.println("cannot happen");
				return;
			case 2:
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve2LR3() {
		int positionIndex;
		int color0 = oct.facetLoc[2][0].face;
		int color1 = oct.facetLoc[3][0].face;
		int startFace = 2, startPositionIndex = 3;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 2LR3 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(3, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
				System.out.println("cannot happen");
				return;
			case 3:
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(2, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve0LR4() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[6][0].face;
		int startFace = 0, startPositionIndex = 4;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR4 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				return;
			case 5:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(startFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve0LR5() {
		int positionIndex;
		int color0 = oct.facetLoc[0][0].face;
		int color1 = oct.facetLoc[6][0].face;
		int startFace = 0, startPositionIndex = 5;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 0LR5 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
			case 4:
				System.out.println("cannot happen");
				return;
			case 5:
				return;
			}
			break;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(6, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve1LR5() {
		int positionIndex;
		int color0 = oct.facetLoc[1][0].face;
		int color1 = oct.facetLoc[5][0].face;
		int startFace = 1, startPositionIndex = 5;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 1LR5 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			System.out.println("cannot happen");
			return;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve3LR4() {
		int positionIndex;
		int color0 = oct.facetLoc[3][0].face;
		int color1 = oct.facetLoc[7][0].face;
		int startFace = 3, startPositionIndex = 4;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 3LR4 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			System.out.println("cannot happen");
			return;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				System.out.println("cannot happen");
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve6LR2() {
		int positionIndex;
		int color0 = oct.facetLoc[6][0].face;
		int color1 = oct.facetLoc[5][0].face;
		int startFace = 6, startPositionIndex = 2;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 6LR2 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			System.out.println("cannot happen");
			return;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				System.out.println("cannot happen");
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				System.out.println("cannot happen");
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 2:
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve6LR3() {
		int positionIndex;
		int color0 = oct.facetLoc[6][0].face;
		int color1 = oct.facetLoc[7][0].face;
		int startFace = 6, startPositionIndex = 3;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 6LR3 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			System.out.println("cannot happen");
			return;
		case 1:
			switch (positionIndex) {
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 5:
				System.out.println("cannot happen");
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				System.out.println("cannot happen");
				return;
			case 5:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				System.out.println("cannot happen");
				return;
			case 3:
				return;
			}
			break;
		}
	}

	void solve1LR4() {
		int positionIndex;
		int color0 = oct.facetLoc[1][0].face;
		int color1 = oct.facetLoc[5][0].face;
		int startFace = 1, startPositionIndex = 4;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 1LR4 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
			System.out.println("cannot happen");
			return;
		case 1:
			switch (positionIndex) {
			case 4:
				return;
			case 5:
				System.out.println("cannot happen");
				return;
			}
			break;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				System.out.println("cannot happen");
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(5, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			}
			break;
		}
	}

	void solve3LR5() {
		int positionIndex;
		int color0 = oct.facetLoc[3][0].face;
		int color1 = oct.facetLoc[7][0].face;
		int startFace = 3, startPositionIndex = 5;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 3LR5 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
			System.out.println("cannot happen");
			return;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 3:
			switch (positionIndex) {
			case 4:
				System.out.println("cannot happen");
				return;
			case 5:
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 5:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(7, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			}
			break;
		}
	}

	void solve6LR0() {
		int positionIndex;
		int color0 = oct.facetLoc[6][0].face;
		int color1 = oct.facetLoc[7][0].face;
		int startFace = 6, startPositionIndex = 0;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 6LR0 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 3:
		case 5:
			System.out.println("cannot happen");
			return;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
				return;
			case 1:
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			}
			break;
		}
	}

	void solve6LR1() {
		int positionIndex;
		int color0 = oct.facetLoc[6][0].face;
		int color1 = oct.facetLoc[5][0].face;
		int startFace = 6, startPositionIndex = 1;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 6LR1 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 3:
		case 5:
			System.out.println("cannot happen");
			return;
		case 2:
			switch (positionIndex) {
			case 0:
			case 1:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 4:
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(4, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 4:
			switch (positionIndex) {
			case 0:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		case 6:
			switch (positionIndex) {
			case 0:
			case 2:
			case 3:
				System.out.println("cannot happen");
				return;
			case 1:
				return;
			}
			break;
		}
	}

	void solve4LR0() {
		int positionIndex;
		int color0 = oct.facetLoc[4][0].face;
		int color1 = oct.facetLoc[5][0].face;
		int startFace = 4, startPositionIndex = 0;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 4LR0 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 2:
		case 3:
		case 5:
		case 6:
			System.out.println("cannot happen");
			return;
		case 4:
			switch (positionIndex) {
			case 0:
				return;
			case 1:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve4LR1() {
		int positionIndex;
		int color0 = oct.facetLoc[4][0].face;
		int color1 = oct.facetLoc[7][0].face;
		int startFace = 4, startPositionIndex = 1;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 4LR1 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 2:
		case 3:
		case 5:
		case 6:
			System.out.println("cannot happen");
			return;
		case 4:
			switch (positionIndex) {
			case 0:
				System.out.println("cannot happen");
				return;
			case 1:
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(4),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			}
			break;
		}
	}

	void solve4LR5() {
		int positionIndex;
		int color0 = oct.facetLoc[4][0].face;
		int color1 = oct.facetLoc[2][0].face;
		int startFace = 4, startPositionIndex = 5;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 4LR5 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 2:
		case 3:
		case 5:
		case 6:
			System.out.println("cannot happen");
			return;
		case 4:
			switch (positionIndex) {
			case 0:
			case 1:
				System.out.println("cannot happen");
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(2),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(5),
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(0),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(1),
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 5:
				return;
			}
			break;
		}
	}

	void solve4LR4() {
		int positionIndex;
		int color0 = oct.facetLoc[4][0].face;
		int color1 = oct.facetLoc[2][0].face;
		int startFace = 4, startPositionIndex = 4;

		returnedFace = startFace;
		returnedPosition = indexToEdgeLR(startPositionIndex);
		findPieceLR(color0, color1);
		positionIndex = edgeLRToIndex(returnedPosition);
		if (DEBUG)
			System.out.println("findPieceLR 4LR4 color0" + color0
				+ " color1=" + color1
				+ " : face=" + returnedFace
				+ " positionIndex=" + positionIndex);
		switch (returnedFace) {
		case 0:
		case 1:
		case 2:
		case 3:
		case 5:
		case 6:
			System.out.println("cannot happen");
			return;
		case 4:
			switch (positionIndex) {
			case 0:
			case 1:
			case 5:
				System.out.println("cannot happen");
				return;
			case 2:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 3:
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(returnedFace, indexToEdgeLR(3),
					OctCanvas.TL, OctInterface.PERIOD4, false);
				return;
			case 4:
				return;
			}
			break;
		}
	}

	void solveFace0() {
		solve0LR0();
		solve0LR1();
		solve0LR2();
		solve0LR3();
	}

	void solveFace2() {
		solve2LR0();
		solve2LR1();
		solve2LR2();
		solve2LR3();
	}

	void solveUpperEdge() {
		solve0LR4();
		solve0LR5();
		solve1LR5();
		solve3LR4();
		solve6LR2();
		solve6LR3();
	}

	void solveLowerEdge() {
		solve1LR4();
		solve3LR5();
		solve6LR0();
		solve6LR1();
	}

	void solveLastEdges() {
		solve4LR0();
		solve4LR1();
		solve4LR5();
		solve4LR4();
	}

	int exactCenter() {
		int size = oct.size;
		int i;

		if (size % 3 == 0)
			size--; /* no exact center here */
		if ((size + 2) % 3 == 0) {
			i = ((size - 1) / 3) * 2;
			return i * i + i;
		} else /*if ((size + 1) % 3 == 0)*/ {
			i = ((size - 2) / 3) * 2 + 1;
			return i * i + i;
		}
	}

	int findPieceCenter(int color0, int startFace) {
		int face;
		int position = exactCenter();

		/* Check starting face first */
		if (oct.facetLoc[startFace][position].face == color0) {
			if (DEBUG)
				System.out.println("Center " + startFace
					+ " " + position);
			return startFace;
		}
		for (face = 0; face < OctCanvas.MAX_FACES; face++) {
			if (face == startFace)
				continue;
			if (oct.facetLoc[face][position].face == color0) {
				if (DEBUG)
					System.out.println("Center fs " + face
						+ " " + position);
				return face;
			}
		}
		System.out.println("Piece " + color0 + " not found!");
		return -1;
	}

	int oppDir(int dir) {
		switch (dir) {
			case OctCanvas.TOP: return OctCanvas.BOTTOM;
			case OctCanvas.TR: return OctCanvas.BL;
			case OctCanvas.RIGHT: return OctCanvas.LEFT;
			case OctCanvas.BR: return OctCanvas.TL;
			case OctCanvas.BOTTOM: return OctCanvas.TOP;
			case OctCanvas.BL: return OctCanvas.TR;
			case OctCanvas.LEFT: return OctCanvas.RIGHT;
			case OctCanvas.TL: return OctCanvas.BR;
			case OctCanvas.CW: return OctCanvas.CCW;
			case OctCanvas.CCW: return OctCanvas.CW;
			default:
				System.out.println("no default " + dir);
				return -1;
		}
	}

	void simpleCenterMove(int face, int dir0, int dir1) {
		int position = exactCenter();
		int i;

		for (i = 0; i < 3; i++) {
			movePuzzlePiece(face, position,
				dir0, OctInterface.PERIOD4, false);
			movePuzzlePiece(face, position,
				dir1, OctInterface.PERIOD4, false);
			movePuzzlePiece(face, position,
				oppDir(dir0), OctInterface.PERIOD4, false);
			movePuzzlePiece(face, position,
				oppDir(dir1), OctInterface.PERIOD4, false);
		}
	}

	void solveFace0Center() {
		int startFace = 0;
		int color0 = oct.facetLoc[startFace][0].face;
		int face = findPieceCenter(color0, startFace);
		int position = exactCenter();

		switch (face) {
		case 0:
				return;
			case 1:
				simpleCenterMove(startFace, OctCanvas.CCW, OctCanvas.TL);
				return;
			case 2:
				simpleCenterMove(1, OctCanvas.TR, OctCanvas.BR);
				return;
			case 3:
				simpleCenterMove(startFace, OctCanvas.CW, OctCanvas.TR);
				return;
			case 4:
				movePuzzlePiece(face, position,
					OctCanvas.TR, OctInterface.PERIOD4, false);
				simpleCenterMove(1, OctCanvas.TR, OctCanvas.BR);
				movePuzzlePiece(face, position,
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 5:
				simpleCenterMove(1, OctCanvas.CW, OctCanvas.BL);
				return;
			case 6:
				simpleCenterMove(startFace, OctCanvas.BR, OctCanvas.BL);
				return;
			case 7:
				simpleCenterMove(3, OctCanvas.CCW, OctCanvas.BR);
				return;
			default:
				System.out.println("not done yet " + face);
				return;
		}
	}

	void solveFace1Center() {
		int startFace = 1;
		int color0 = oct.facetLoc[startFace][0].face;
		int face = findPieceCenter(color0, startFace);
		int position = exactCenter();

		switch (face) {
			case 0:
				System.out.println("should not happen " + face);
				return;
			case 1:
				return;
			case 2:
				simpleCenterMove(face, OctCanvas.CCW, OctCanvas.TR);
				return;
			case 3:
				simpleCenterMove(2, OctCanvas.BL, OctCanvas.BR);
				return;
			case 4:
				simpleCenterMove(2, OctCanvas.CW, OctCanvas.TL);
				return;
			case 5:
				simpleCenterMove(face, OctCanvas.TR, OctCanvas.BR);
				return;
			case 6:
				simpleCenterMove(4, OctCanvas.CCW, OctCanvas.TL);
				return;
			case 7:
				movePuzzlePiece(face, position,
					OctCanvas.TL, OctInterface.PERIOD4, false);
				simpleCenterMove(2, OctCanvas.BR, OctCanvas.BL);
				movePuzzlePiece(face, position,
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			default:
				System.out.println("not done yet " + face);
				return;
		}
	}

	void solveFace3Center() {
		int startFace = 3;
		int color0 = oct.facetLoc[startFace][0].face;
		int face = findPieceCenter(color0, startFace);
		int position = exactCenter();

		switch (face) {
			case 0:
			case 1:
				System.out.println("should not happen " + face);
				return;
			case 2:
				movePuzzlePiece(face, position,
					OctCanvas.TR, OctInterface.PERIOD4, false);
				simpleCenterMove(face, OctCanvas.BR, OctCanvas.BL);
				movePuzzlePiece(face, position,
					OctCanvas.BL, OctInterface.PERIOD4, false);
				return;
			case 3:
				return;
			case 4:
				simpleCenterMove(6, OctCanvas.CCW, OctCanvas.BR);
				return;
			case 5:
				movePuzzlePiece(face, position,
					OctCanvas.BL, OctInterface.PERIOD4, false);
				simpleCenterMove(4, OctCanvas.CW, OctCanvas.TR);
				movePuzzlePiece(face, position,
					OctCanvas.TR, OctInterface.PERIOD4, false);
				return;
			case 6:
				simpleCenterMove(4, OctCanvas.CW, OctCanvas.TR);
				return;
			case 7:
				simpleCenterMove(face, OctCanvas.TL, OctCanvas.BL);
				return;
			default:
				System.out.println("not done yet " + face);
				return;
		}
	}

	void solveFace6Center() {
		int startFace = 6;
		int color0 = oct.facetLoc[startFace][0].face;
		int face = findPieceCenter(color0, startFace);
		int position = exactCenter();

		switch (face) {
			case 0:
			case 1:
			case 3:
				System.out.println("should not happen " + face);
				return;
			case 2:
				movePuzzlePiece(face, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(face, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				simpleCenterMove(startFace, OctCanvas.BR, OctCanvas.BL);
				movePuzzlePiece(face, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(face, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			case 4:
				movePuzzlePiece(face, position,
					OctCanvas.TL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				simpleCenterMove(face, OctCanvas.CW, OctCanvas.TR);
				movePuzzlePiece(2, position,
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(face, position,
					OctCanvas.BR, OctInterface.PERIOD4, false);
				return;
			case 5:
				movePuzzlePiece(2, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				movePuzzlePiece(face, position,
					OctCanvas.TL, OctInterface.PERIOD4, false);
				simpleCenterMove(7, OctCanvas.TL, OctCanvas.BL);
				movePuzzlePiece(face, position,
					OctCanvas.BR, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, position,
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				return;
			case 6:
				return;
			case 7:
				movePuzzlePiece(2, position,
					OctCanvas.CCW, OctInterface.PERIOD4, false);
				movePuzzlePiece(face, position,
					OctCanvas.TR, OctInterface.PERIOD4, false);
				simpleCenterMove(5, OctCanvas.TR, OctCanvas.BR);
				movePuzzlePiece(face, position,
					OctCanvas.BL, OctInterface.PERIOD4, false);
				movePuzzlePiece(2, position,
					OctCanvas.CW, OctInterface.PERIOD4, false);
				return;
			default:
				System.out.println("not done yet " + face);
				return;
		}
	}

	void lastCenters() {
		int[] bgColors = new int[OctCanvas.MAX_FACES];
		int[] centerColors = new int[OctCanvas.MAX_FACES];
		int[/*4*/] lastColors = {2, 4, 5, 7};
		int[] colorSequence = new int[4];
		int position = exactCenter();
		int i, j;

		for (i = 0; i < 4; i++) {
			int lastColor = lastColors[i];
			bgColors[lastColor] =
				oct.facetLoc[lastColor][0].face;
			centerColors[lastColor] =
				oct.facetLoc[lastColor][position].face;
			colorSequence[i] = -1;
		}
		for (i = 0; i < 4; i++) {
			int bgColor = bgColors[lastColors[i]];
			for (j = 0; j < 4; j++) {
				if (bgColor == centerColors[lastColors[j]]) {
					colorSequence[i] = j;
				}
			}
		}
		/* 12 possibilities */
		if (colorSequence[0] == 0 && colorSequence[1] == 1 &&
				colorSequence[2] == 2 && colorSequence[3] == 3) {
			return;
		} else if (colorSequence[0] == 0 && colorSequence[1] == 2 &&
				colorSequence[2] == 3 && colorSequence[3] == 1) {
			/* 0231 */
			movePuzzlePiece(5, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
			movePuzzlePiece(5, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 0 && colorSequence[1] == 3 &&
				colorSequence[2] == 1 && colorSequence[3] == 2) {
			/* 0312 */
			movePuzzlePiece(7, position,
				OctCanvas.BR, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CW, OctCanvas.BR);
			movePuzzlePiece(7, position,
				OctCanvas.TL, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.BR, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.TL, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 1 && colorSequence[1] == 0 &&
				colorSequence[2] == 3 && colorSequence[3] == 2) {
			/* 1032 */
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
		} else if (colorSequence[0] == 1 && colorSequence[1] == 2 &&
				colorSequence[2] == 0 && colorSequence[3] == 3) {
			/* 1203 */
			movePuzzlePiece(5, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(5, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CW, OctCanvas.BR);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 1 && colorSequence[1] == 3 &&
				colorSequence[2] == 2 && colorSequence[3] == 0) {
			/* 1320 */
			movePuzzlePiece(6, position,
				OctCanvas.BR, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(6, position,
				OctCanvas.TL, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 2 && colorSequence[1] == 0 &&
				colorSequence[2] == 1 && colorSequence[3] == 3) {
			/* 2013 */
			movePuzzlePiece(4, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
			movePuzzlePiece(4, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.BR, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CW, OctCanvas.BR);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.TL, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 2 && colorSequence[1] == 1 &&
				colorSequence[2] == 3 && colorSequence[3] == 0) {
			/* 2130 */
			movePuzzlePiece(6, position,
				OctCanvas.BR, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
			movePuzzlePiece(6, position,
				OctCanvas.TL, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 2 && colorSequence[1] == 3 &&
				colorSequence[2] == 0 && colorSequence[3] == 1) {
			/* 2301 */
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
		} else if (colorSequence[0] == 3 && colorSequence[1] == 0 &&
				colorSequence[2] == 2 && colorSequence[3] == 1) {
			/* 3021 */
			movePuzzlePiece(4, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(4, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			movePuzzlePiece(4, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CCW, OctCanvas.BL);
			movePuzzlePiece(4, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 3 && colorSequence[1] == 1 &&
				colorSequence[2] == 0 && colorSequence[3] == 2) {
			/* 3102 */
			movePuzzlePiece(5, position,
				OctCanvas.BL, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.CW, OctCanvas.BR);
			movePuzzlePiece(5, position,
				OctCanvas.TR, OctInterface.PERIOD4, false);
			movePuzzlePiece(2, position,
				OctCanvas.CW, OctInterface.PERIOD4, false);
			simpleCenterMove(4, OctCanvas.TR, OctCanvas.TL);
			movePuzzlePiece(2, position,
				OctCanvas.CCW, OctInterface.PERIOD4, false);
		} else if (colorSequence[0] == 3 && colorSequence[1] == 2 &&
				colorSequence[2] == 1 && colorSequence[3] == 0) {
			/* 3210 */
			simpleCenterMove(4, OctCanvas.CW, OctCanvas.BR);
		} else {
			/* seems like more possibilities when size 5 */
			System.out.println(colorSequence[0] + " " +
				colorSequence[1] + " " +
				colorSequence[2] + " " +
				colorSequence[3] + " center not done");
		}
	}


	void centerRotate123(int face, int dir0, int dir1, int dir2) {
		int i;
		int position = exactCenter();

		for (i = 0; i < 28; i++) {
			movePuzzlePiece(face, position,
				dir0, OctInterface.PERIOD4, false);
			movePuzzlePiece(face, position,
				dir1, OctInterface.PERIOD4, false);
			movePuzzlePiece(face, position,
				dir2, OctInterface.PERIOD4, false);
		}
	}

	void centerRotate4(int dir) {
		switch (dir) {
		case OctCanvas.TR:
			centerRotate123(4, OctCanvas.CW, OctCanvas.BL, OctCanvas.TL); /* / OctCanvas.CW */
			return;
		case OctCanvas.BL:
			centerRotate123(4, OctCanvas.BR, OctCanvas.TR, OctCanvas.CCW); /* / OctCanvas.CCW */
			return;
		case OctCanvas.BR:
			centerRotate123(4, OctCanvas.BL, OctCanvas.TL, OctCanvas.CW); /* \ OctCanvas.CW */
			return;
		case OctCanvas.TL:
			centerRotate123(4, OctCanvas.CCW, OctCanvas.BR, OctCanvas.TR); /* \ OctCanvas.CCW */
			return;
		case OctCanvas.RIGHT: 
IGHT:
			centerRotate123(4, OctCanvas.TL, OctCanvas.CW, OctCanvas.BL); /* - OctCanvas.CW */
			return;
		case OctCanvas.LEFT:
			centerRotate123(4, OctCanvas.TR, OctCanvas.CCW, OctCanvas.BR); /* - OctCanvas.CCW */
			return;
		default:
			break;
		}
	}

	int findPieceCenterOrient(int face)
	{
		int position = exactCenter();

		return (oct.facetLoc[face][position].rotation + OctCanvas.MAX_ORIENT -
			oct.facetLoc[face][0].rotation) % 3;
	}

	/*void checkOrient()
	{
		int i, value;
		for (i = 0; i < 8; i++) {
			value = findPieceCenterOrient(i);
			println(i + ": " + value);
		}
	
	}*/

	void rotateFirstHalf() {
		int face = 0;
		int value = findPieceCenterOrient(face);

		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println(face + " " + value);
			if (opp) {
				centerRotate123(face, OctCanvas.TR, OctCanvas.CCW, OctCanvas.BR);
			} else {
				centerRotate123(face, OctCanvas.TL, OctCanvas.CW, OctCanvas.BL);
			}
		}
		face = 3;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println(face + " " + value);
			if (opp) {
				centerRotate123(2, OctCanvas.CCW, OctCanvas.TL, OctCanvas.BL);
			} else {
				centerRotate123(2, OctCanvas.TR, OctCanvas.BR, OctCanvas.CW);
			}
		}
		face = 1;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println(face + " " + value);
			if (opp) {
				centerRotate123(2, OctCanvas.TL, OctCanvas.BL, OctCanvas.CCW);
			} else {
				centerRotate123(2, OctCanvas.CW, OctCanvas.TR, OctCanvas.BR);
			}
		}
		face = 6;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println(face + " " + value);
			if (opp) {
				centerRotate123(face, OctCanvas.BL, OctCanvas.CCW, OctCanvas.TL);
			} else {
				centerRotate123(face, OctCanvas.BR, OctCanvas.CW, OctCanvas.TR);
			}
		}
	}

	void rotateLastHalf() {
		int[/*4*/] lastOrients = {2, 4, 5, 7};
		int[] orientSequence = new int[4];
		int i;

		for (i = 0; i < 4; i++)
			orientSequence[i] = findPieceCenterOrient(lastOrients[i]);
		if (DEBUG)
			System.out.println("last half " +
				orientSequence[1] + " " + orientSequence[0] + " " +
				orientSequence[2] + " " +  orientSequence[3]);
		if (orientSequence[1] == 0 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 0) {
			return;
		} else if (orientSequence[1] == 0 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 2) {
			/* 0012 1122 2202 */
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.TR);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 1) {
			/* 0021 1122 2220 */
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.BR);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 2) {
			/* 0102 1212 2022 */
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 1) {
			/* 0111 1221 2022 */
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 0) {
			/* 0120 1221 2022 */
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 1) {
			/* 0201 1212 2022 */
			centerRotate4(OctCanvas.RIGHT);
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 0) {
			/* 0210 1212 2022 */
			centerRotate4(OctCanvas.RIGHT);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 0 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 2) {
			/* 0222 2112 1011 */
			centerRotate4(OctCanvas.TL);
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 2) {
			/* 1002 2112 1011 */
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 1) {
			/* 1011 */
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 0) {
			/* 1020 2121 1011 */
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.TL);
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 1) {
			/* 1101 */
			centerRotate4(OctCanvas.BL);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 0) {
			/* 1110 */
			centerRotate4(OctCanvas.TL);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 2) {
			/* 1122 2220 */
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.BR);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 0) {
			/* 1200 2211 1110 */
			centerRotate4(OctCanvas.RIGHT);
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.TL);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 2) {
			/* 1212 2022 */
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 1 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 1) {
			/* 1221 2022 */
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 1) {
			/* 2001 1221 2022 */
			centerRotate4(OctCanvas.TL);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 0) {
			/* 2010 1212 2022 */
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.BR);
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 0 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 2) {
			/* 2022 */
			centerRotate4(OctCanvas.RIGHT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 0) {
			/* 2100 1122 2220 */
			centerRotate4(OctCanvas.LEFT);
			centerRotate4(OctCanvas.TR);
			centerRotate4(OctCanvas.BR);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 2) {
			/* 2112 1011 */
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 1 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 1) {
			/* 2121 1011 */
			centerRotate4(OctCanvas.TL);
			centerRotate4(OctCanvas.LEFT);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 0 && orientSequence[3] == 2) {
			/* 2202 */
			centerRotate4(OctCanvas.TR);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 1 && orientSequence[3] == 1) {
			/* 2211 1110 */
			centerRotate4(OctCanvas.BL);
			centerRotate4(OctCanvas.TL);
		} else if (orientSequence[1] == 2 && orientSequence[0] == 2 &&
	                        orientSequence[2] == 2 && orientSequence[3] == 0) {
			/* 2220 */
			centerRotate4(OctCanvas.BR);
		}
		if (DEBUG) {
			for (i = 0; i < 4; i++)
				orientSequence[i] = findPieceCenterOrient(lastOrients[i]);
			System.out.println("final " +
				orientSequence[1] + " " + orientSequence[0] + " " +
				orientSequence[2] + " " +  orientSequence[3]);
		}
	}

	void solveCorners() {
		if (oct.size > 2) {
			solveTopFace();
			solveSecondFace();
			solveThirdCorner();
			lastCount = 0;
			solveLastCorner();
			if (oct.size > 3) {
				solveFace0();
				solveFace2();
				solveUpperEdge();
				solveLowerEdge();
				solveLastEdges();
				if (oct.size % 3 != 0) {
					solveFace0Center();
					solveFace1Center();
					solveFace3Center();
					solveFace6Center();
					lastCenters();
					if (oct.orient) {
						rotateFirstHalf();
						rotateLastHalf();
					}
				}
			}
		} else {
			allTrivialCorners();
		}
	}

	static boolean
	checkCornerPiece(int color, int face, int position) {
		int newFace, newPosition, positionIndex;

		positionIndex = cornerToIndex(position);
		if (positionIndex == -1) {
			positionIndex = 0;
			System.out.println("position " + position + " incorrect");
		}
		newFace = edgeMateCorner[face][positionIndex];
		newPosition = indexToCornerMate(positionIndex);
		return (oct.facetLoc[newFace][newPosition].face == color);
	}

	static void
	findCornerPiece(int color0, int color1) {
		// returns returnedFace, returnedPosition;
		int positionIndex;

		for (returnedFace = 0; returnedFace < OctCanvas.MAX_FACES; (returnedFace)++) {
			for (positionIndex = 0; positionIndex < 3; positionIndex++) {
				returnedPosition = indexToCorner(positionIndex);
				if (oct.facetLoc[returnedFace][returnedPosition].face == color0 &&
						checkCornerPiece(color1, returnedFace, returnedPosition)) {
					return;
				}
			}
		}
		System.out.println("Corner piece " + color0 + " " + color1 + " not found!");
	}

	void solveTRCorner() {
		/* Pick a face (0) and determine which color is present at each
		 * corner. */
		int topColor = oct.facetLoc[0][0].face;
		int otherColor = oct.facetLoc[1][0].face;
		findCornerPiece(topColor, otherColor);
		if (DEBUG)
			System.out.println("TR face "
				+ returnedFace + ", position " + returnedPosition);
		if (returnedFace == 0 && returnedPosition == indexToCorner(1))
			return;
		else if (returnedFace == 0 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 2 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (returnedFace == 2 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 5 && returnedPosition == indexToCorner(0)) {
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 5 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 5 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 7 && returnedPosition == indexToCorner(0)) {
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (returnedFace == 7 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (returnedFace == 7 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}

	void solveTLCorner() {
		int topColor = oct.facetLoc[0][0].face;
		int otherColor = oct.facetLoc[3][0].face;
		findCornerPiece(otherColor, topColor);
		if (DEBUG)
			System.out.println("TL face "
				+ returnedFace + ", position " + returnedPosition);
		if (returnedFace == 1 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (returnedFace == 3 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (returnedFace == 3 && returnedPosition == indexToCorner(1)) {
			return;
		} else if (returnedFace == 4 && returnedPosition == indexToCorner(0)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (returnedFace == 4 && returnedPosition == indexToCorner(2)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (returnedFace == 4 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (returnedFace == 6 && returnedPosition == indexToCorner(0)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (returnedFace == 6 && returnedPosition == indexToCorner(1)) {
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
	}
	void solveTopCorners() {
		solveTRCorner();
		solveTLCorner();
	}

	void rotateLeftRightCorners () {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	static int lastColor() {
		int[] faces = new int[8];
		int i, j;

		faces[0] = oct.facetLoc[0][indexToCorner(0)].face;
		faces[1] = oct.facetLoc[1][indexToCorner(0)].face;
		faces[2] = oct.facetLoc[2][indexToCorner(0)].face;
		faces[3] = oct.facetLoc[3][indexToCorner(0)].face;
		faces[4] = oct.facetLoc[5][indexToCorner(1)].face;
		faces[5] = oct.facetLoc[6][indexToCorner(2)].face;
		faces[6] = oct.facetLoc[7][indexToCorner(2)].face;
		faces[7] = -1;
		for (i = 0; i < 8; i++) {
			for (j = 0; j < 7; j++) {
				if (i == faces[j])
					break;
			}
			if (i != faces[j])
				return i;
		}
		return -1;
	}

	void solveBottomCorners() {
		int bottomColor = lastColor();
		int rightColor = oct.facetLoc[5][indexToCorner(1)].face;
		int leftColor = oct.facetLoc[7][indexToCorner(2)].face;
		int[] face = new int[3];
		int[] position = new int[3];

		findCornerPiece(bottomColor, leftColor);
		face[0] = returnedFace;
		position[0] = returnedPosition;
		if (DEBUG)
			System.out.println("OctCanvas.BOTTOM face "
				+ face[0] + ", position " + position[0]);
		findCornerPiece(bottomColor, rightColor);
		face[1] = returnedFace;
		position[1] = returnedPosition;
		if (DEBUG)
			System.out.println("OctCanvas.RIGHT face "
				+ face[1] + ", position " + position[1]);
		findCornerPiece(leftColor, bottomColor);
		face[2] = returnedFace;
		position[2] = returnedPosition;
		if (DEBUG)
			System.out.println("OctCanvas.LEFT face "
				+ face[2] + ", position " + position[2]);
		if (face[0] == 1 && position[0] == indexToCorner(1) &&
				face[1] == 3 && position[1] ==  indexToCorner(2)&&
				face[2] == 5 && position[2] == indexToCorner(0)) {
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face[0] == 1 && position[0] == indexToCorner(1) &&
				face[1] == 4 && position[1] == indexToCorner(2) &&
				face[2] == 7 && position[2] == indexToCorner(0)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face[0] == 3 && position[0] == indexToCorner(2) &&
				face[1] == 4 && position[1] == indexToCorner(0) &&
				face[2] == 5 && position[2] == indexToCorner(2)) {
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face[0] == 3 && position[0] == indexToCorner(2) &&
				face[1] == 6 && position[1] == indexToCorner(0) &&
				face[2] == 2 && position[2] == indexToCorner(2)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face[0] == 4 && position[0] == indexToCorner(0) &&
				face[1] == 1 && position[1] == indexToCorner(1) &&
				face[2] == 2 && position[2] == indexToCorner(1)) {
			rotateLeftRightCorners();
		} else if (face[0] == 4 && position[0] == indexToCorner(2) &&
				face[1] == 4 && position[1] == indexToCorner(0) &&
				face[2] == 2 && position[2] == indexToCorner(2)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face[0] == 4 && position[0] == indexToCorner(1) &&
				face[1] == 3 && position[1] == indexToCorner(2) &&
				face[2] == 7 && position[2] == indexToCorner(0)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
		} else if (face[0] == 4 && position[0] == indexToCorner(1) &&
				face[1] == 4 && position[1] == indexToCorner(2) &&
				face[2] == 5 && position[2] == indexToCorner(0)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face[0] == 4 && position[0] == indexToCorner(0) &&
				face[1] == 4 && position[1] == indexToCorner(1) &&
				face[2] == 7 && position[2] == indexToCorner(1)) {
			return;
		} else if (face[0] == 4 && position[0] == indexToCorner(2) &&
				face[1] == 6 && position[1] == indexToCorner(0) &&
				face[2] == 5 && position[2] == indexToCorner(2)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
		} else if (face[0] == 6 && position[0] == indexToCorner(0) &&
				face[1] == 1 && position[1] == indexToCorner(1) &&
				face[2] == 7 && position[2] == indexToCorner(1)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face[0] == 6 && position[0] == indexToCorner(0) &&
				face[1] == 4 && position[1] == indexToCorner(1) &&
				face[2] == 2 && position[2] == indexToCorner(1)) {
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			rotateLeftRightCorners();
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		}
	}

	void rotateAroundCenter(int face, int direction) {
		int i;

		switch (face) {
		case 0:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(0, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(0, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, true);
			}
			break;
		case 1:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(1, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(1, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, true);
			}
			break;
		case 2:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, true);
			}
			break;
		case 3:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(3, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(3, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, true);
			}
			break;
		case 4:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, true);
			}
			break;
		case 5:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(5, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, true);
			}
			break;
		case 6:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(0, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(6, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(0, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, true);
			}
			break;
		case 7:
			if (direction == OctCanvas.CW) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(7, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, true);
			} else {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(7, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(2, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(0, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				}
				movePuzzlePiece(7, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, true);
			}
			break;
		}
	}

	void rotate4Centers(int nextFace) {
		int i;

		switch (nextFace) {
		case 2:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			}
			break;
		case 3:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			}
			break;
		case 4:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
			}
			break;
		case 5:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
			}
			break;
		case 6:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(7, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
			}
			break;
		case 7:
			for (i = 0; i < 3; i++) {
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(3, indexToCorner(0),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(1, indexToCorner(0),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
			}
			break;
		}
	}

	void solveTopEvenBottomOddCenters() {
		int[] color = new int[4];
		int[] face = new int[4];
		int i;

		color[0] = oct.facetLoc[0][0].face;
		color[1] = oct.facetLoc[2][0].face;
		color[2] = oct.facetLoc[5][0].face;
		color[3] = oct.facetLoc[7][0].face;

		for (i = 0; i < 4; i++) {
			face[i] = findPieceCenter(color[i], 0);
		}
		if (DEBUG)
			System.out.println("solveTopEvenBottomOddCenters "
				+ face[0] + " " + face[1] + " "
				+ face[2] + " " + face[3]);
		if (face[0] == 0 && face[1] == 2
				&& face[2] == 5 && face[3] == 7) {
			return;
		} else if (face[0] == 0 && face[1] == 5
				&& face[2] == 7 && face[3] == 2) {
			rotateAroundCenter(4, OctCanvas.CCW);
			return;
		} else if (face[0] == 0 && face[1] == 7
				&& face[2] == 2 && face[3] == 5) {
			rotateAroundCenter(4, OctCanvas.CW);
			return;
		} else if (face[0] == 2 && face[1] == 0
				&& face[2] == 7 && face[3] == 5) {
			rotate4Centers(2);
			return;
		} else if (face[0] == 2 && face[1] == 5
				&& face[2] == 0 && face[3] == 7) {
			rotateAroundCenter(1, OctCanvas.CW);
			return;
		} else if (face[0] == 2 && face[1] == 7
				&& face[2] == 5 && face[3] == 0) {
			rotateAroundCenter(3, OctCanvas.CCW);
			return;
		} else if (face[0] == 5 && face[1] == 0
				&& face[2] == 2 && face[3] == 7) {
			rotateAroundCenter(1, OctCanvas.CCW);
			return;
		} else if (face[0] == 5 && face[1] == 2
				&& face[2] == 7 && face[3] == 0) {
			rotateAroundCenter(6, OctCanvas.CW);
			return;
		} else if (face[0] == 5 && face[1] == 7
				&& face[2] == 0 && face[3] == 2) {
			rotate4Centers(5);
			return;
		} else if (face[0] == 7 && face[1] == 0
				&& face[2] == 5 && face[3] == 2) {
			rotateAroundCenter(3, OctCanvas.CW);
			return;
		} else if (face[0] == 7 && face[1] == 2
				&& face[2] == 0 && face[3] == 5) {
			rotateAroundCenter(6, OctCanvas.CCW);
			return;
		} else if (face[0] == 7 && face[1] == 5
				&& face[2] == 2 && face[3] == 0) {
			rotate4Centers(7);
			return;
		}
	}

	void solveTopOddBottomEvenCenters() {
		int[] color = new int[4];
		int[] face = new int[4];
		int i;

		color[0] = oct.facetLoc[1][0].face;
		color[1] = oct.facetLoc[3][0].face;
		color[2] = oct.facetLoc[4][0].face;
		color[3] = oct.facetLoc[6][0].face;

		for (i = 0; i < 4; i++) {
			face[i] = findPieceCenter(color[i], 0);
		}
		if (DEBUG)
			System.out.println("solveTopOddBottomEvenCenters"
				+ face[0] + " " + face[1] + " "
				+ face[2] + " " + face[3]);
		if (face[0] == 1 && face[1] == 3
				&& face[2] == 4 && face[3] == 6) {
			return;
		} else if (face[0] == 1 && face[1] == 4
				&& face[2] == 6 && face[3] == 3) {
			rotateAroundCenter(7, OctCanvas.CCW);
			return;
		} else if (face[0] == 1 && face[1] == 6
				&& face[2] == 3 && face[3] == 4) {
			rotateAroundCenter(7, OctCanvas.CW);
			return;
		} else if (face[0] == 3 && face[1] == 1
				&& face[2] == 6 && face[3] == 4) {
			rotate4Centers(3);
			return;
		} else if (face[0] == 3 && face[1] == 4
				&& face[2] == 1 && face[3] == 6) {
			rotateAroundCenter(2, OctCanvas.CW);
			return;
		} else if (face[0] == 3 && face[1] == 6
				&& face[2] == 4 && face[3] == 1) {
			rotateAroundCenter(0, OctCanvas.CCW);
			return;
		} else if (face[0] == 4 && face[1] == 1
				&& face[2] == 3 && face[3] == 6) {
			rotateAroundCenter(2, OctCanvas.CCW);
			return;
		} else if (face[0] == 4 && face[1] == 3
				&& face[2] == 6 && face[3] == 1) {
			rotateAroundCenter(5, OctCanvas.CW);
			return;
		} else if (face[0] == 4 && face[1] == 6
				&& face[2] == 1 && face[3] == 3) {
			rotate4Centers(4);
			return;
		} else if (face[0] == 6 && face[1] == 1
				&& face[2] == 4 && face[3] == 3) {
			rotateAroundCenter(0, OctCanvas.CW);
			return;
		} else if (face[0] == 6 && face[1] == 3
				&& face[2] == 1 && face[3] == 4) {
			rotateAroundCenter(5, OctCanvas.CCW);
			return;
		} else if (face[0] == 6 && face[1] == 4
				&& face[2] == 3 && face[3] == 1) {
			rotate4Centers(6);
			return;
		}
	}

	void rotateTopHalf() {
		int face = 0;
		int value = findPieceCenterOrient(face);

		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println("rotateTopHalf "
					+ face + " " + value);
			if (opp) {
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
			} else {
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.TOP, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(5, indexToCorner(2),
					OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				movePuzzlePiece(6, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			}
		}
		face = 3;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println("rotateTopHalf "
					+ face + " " + value);
			if (opp) {
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			} else {
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			}
		}
		face = 1;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println("rotateTopHalf "
					+ face + " " + value);
			if (opp) {
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
			} else {
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.RIGHT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(4, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
				movePuzzlePiece(2, indexToCorner(2),
					OctCanvas.LEFT, OctInterface.PERIOD3, false);
			}
		}
		face = 2;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			int i;
			boolean opp = (value == 1);
			if (DEBUG)
				System.out.println("rotateTopHalf "
					+ face + " " + value);
			if (opp) {
				for (i = 0; i < 10; i++) {
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
			} else {
				for (i = 0; i < 10; i++) {
					movePuzzlePiece(4, indexToCorner(2),
						OctCanvas.LEFT, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToCorner(2),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToCorner(2),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
			}
		}
	}

	void bottomTriangleTwister(int direction) {
		switch (direction) {
		case OctCanvas.TR:
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			break;
		case OctCanvas.BR:
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			break;
		case OctCanvas.BL:
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			break;
		case OctCanvas.TL:
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToCorner(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToCorner(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			break;
		}
	}

	void rotateBottomHalf() {
		int[] value = new int[4];
		int i;

		for (i = 0; i < 4; i++)
			value[i] = findPieceCenterOrient(i + 4);
		if (DEBUG)
			System.out.println("rotateBottomHalf "
				+ value[0] + value[1] + value[2] + value[3]);
		if (value[0] == 0 && value[1] == 0 &&
				value[2] == 0 && value[3] == 0) {
			return;
		} else if (value[0] == 0 && value[1] == 1 &&
				value[2] == 0 && value[3] == 2) {
			bottomTriangleTwister(OctCanvas.TR);
			bottomTriangleTwister(OctCanvas.BR);
		} else if (value[0] == 0 && value[1] == 2 &&
				value[2] == 0 && value[3] == 1) {
			bottomTriangleTwister(OctCanvas.TL);
			bottomTriangleTwister(OctCanvas.BL);
		} else if (value[0] == 1 && value[1] == 0 &&
				value[2] == 2 && value[3] == 0) {
			bottomTriangleTwister(OctCanvas.TR);
			bottomTriangleTwister(OctCanvas.TL);
		} else if (value[0] == 1 && value[1] == 1 &&
				value[2] == 2 && value[3] == 2) {
			bottomTriangleTwister(OctCanvas.BL);
		} else if (value[0] == 1 && value[1] == 2 &&
				value[2] == 2 && value[3] == 1) {
			bottomTriangleTwister(OctCanvas.BR);
		} else if (value[0] == 2 && value[1] == 0 &&
				value[2] == 1 && value[3] == 0) {
			bottomTriangleTwister(OctCanvas.BR);
			bottomTriangleTwister(OctCanvas.BL);
		} else if (value[0] == 2 && value[1] == 1 &&
			value[2] == 1 && value[3] == 2) {
			bottomTriangleTwister(OctCanvas.TL);
		} else if (value[0] == 2 && value[1] == 2 &&
				value[2] == 1 && value[3] == 1) {
			bottomTriangleTwister(OctCanvas.TR);
		}
	}

	void simpleEdgeMove(int face0, int dir0, int face1, int dir1) {
		int position = indexToCorner(2);
		int i;
	
		for (i = 0; i < 4; i++) {
			movePuzzlePiece(face0, position,
				dir0, OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, position,
				dir1, OctInterface.PERIOD3, false);
			movePuzzlePiece(face0, position,
				oppDir(dir0), OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, position,
				oppDir(dir1), OctInterface.PERIOD3, false);
		}
	}
	
	void edge0Pos2(int face, int position) {
		if (face == 1 && edgeToIndex(position) == 0) {
			movePuzzlePiece(1, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 1 && edgeToIndex(position) == 1) {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 1 && edgeToIndex(position) == 2) {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 0) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 1) {
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 2) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 0) {
			movePuzzlePiece(5, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 1) {
			movePuzzlePiece(7, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(1),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 0) {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 1) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(1),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 2) {
			return;
		}
	}
	 
	void edge0Pos0(int face, int position) {
		if (face == 1 && edgeToIndex(position) == 0) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 1 && edgeToIndex(position) == 1) {
			return;
		} else if (face == 1 && edgeToIndex(position) == 2) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 0) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 1) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 2) {
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 0) {
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 1) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 0) {
			movePuzzlePiece(6, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(6, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 1) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 2) {
				System.out.println("cannot get here\n");
		}
	}
	
	void edge0Pos1(int face, int position) {
		if (face == 1 && edgeToIndex(position) == 0) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(7, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 1 && edgeToIndex(position) == 1) {
				System.out.println("cannot get here\n");
		} else if (face == 1 && edgeToIndex(position) == 2) {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(7, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 0) {
			return;
		} else if (face == 3 && edgeToIndex(position) == 1) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(7, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 3 && edgeToIndex(position) == 2) {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 0) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(7, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 1) {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(1),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 0) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(1),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 1) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(7, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 2) {
				System.out.println("cannot get here\n");
		}
	}
	
	void edge1Pos2(int face, int position) {
		if (face == 2 && edgeToIndex(position) == 0) {
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 2 && edgeToIndex(position) == 1) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 2 && edgeToIndex(position) == 2) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 5 && edgeToIndex(position) == 0) {
			simpleEdgeMove(0, OctCanvas.LEFT, 1, OctCanvas.TOP);
		} else if (face == 5 && edgeToIndex(position) == 1) {
			simpleEdgeMove(2, OctCanvas.LEFT, 1, OctCanvas.BOTTOM);
		} else if (face == 5 && edgeToIndex(position) == 2) {
			return;	
		} else if (face == 7 && edgeToIndex(position) == 0) {
			simpleEdgeMove(1, OctCanvas.BOTTOM, 2, OctCanvas.LEFT);
		} else if (face == 7 && edgeToIndex(position) == 1) {
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 7 && edgeToIndex(position) == 2) {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleEdgeMove(1, OctCanvas.TOP, 0, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}
	
	void edge3Pos2(int face, int position) {
		if (face == 2 && edgeToIndex(position) == 0) {
			simpleEdgeMove(4, OctCanvas.RIGHT, 7, OctCanvas.TOP);
		} else if (face == 2 && edgeToIndex(position) == 1) {
			simpleEdgeMove(7, OctCanvas.TOP, 4, OctCanvas.RIGHT);
		} else if (face == 2 && edgeToIndex(position) == 2) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(3, OctCanvas.BOTTOM, 2, OctCanvas.RIGHT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 5 && edgeToIndex(position) == 0) {
			simpleEdgeMove(3, OctCanvas.TOP, 0, OctCanvas.RIGHT);
		} else if (face == 5 && edgeToIndex(position) == 1) {
			simpleEdgeMove(3, OctCanvas.BOTTOM, 2, OctCanvas.RIGHT);
		} else if (face == 7 && edgeToIndex(position) == 0) {
			simpleEdgeMove(2, OctCanvas.RIGHT, 3, OctCanvas.BOTTOM);
		} else if (face == 7 && edgeToIndex(position) == 1) {
			simpleEdgeMove(0, OctCanvas.RIGHT, 3, OctCanvas.TOP);
		} else if (face == 7 && edgeToIndex(position) == 2) {
			return;
		}
	}
	
	void edge1Pos0(int face, int position) {
		if (face == 2 && edgeToIndex(position) == 0) {
			simpleEdgeMove(2, OctCanvas.LEFT, 4, OctCanvas.RIGHT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.LEFT, 2, OctCanvas.RIGHT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 2 && edgeToIndex(position) == 1) {
			return;
		} else if (face == 2 && edgeToIndex(position) == 2) {
			simpleEdgeMove(2, OctCanvas.RIGHT, 4, OctCanvas.LEFT);
		} else if (face == 5 && edgeToIndex(position) == 0) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.LEFT, 2, OctCanvas.RIGHT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 5 && edgeToIndex(position) == 1) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.LEFT, 2, OctCanvas.RIGHT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 7 && edgeToIndex(position) == 0) {
			simpleEdgeMove(4, OctCanvas.LEFT, 2, OctCanvas.RIGHT);
		} else if (face == 7 && edgeToIndex(position) == 1) {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.LEFT, 2, OctCanvas.RIGHT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}
	
	void edge3Pos1(int face, int position) {
		if (face == 2 && edgeToIndex(position) == 0) {
			return;
		} else if (face == 2 && edgeToIndex(position) == 2) {
			simpleEdgeMove(2, OctCanvas.LEFT, 4, OctCanvas.RIGHT);
		} else if (face == 5 && edgeToIndex(position) == 0) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.RIGHT, 2, OctCanvas.LEFT);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (face == 5 && edgeToIndex(position) == 1) {
			simpleEdgeMove(4, OctCanvas.RIGHT, 2, OctCanvas.LEFT);
		} else if (face == 7 && edgeToIndex(position) == 0) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.RIGHT, 2, OctCanvas.LEFT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 7 && edgeToIndex(position) == 1) {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(4, OctCanvas.RIGHT, 2, OctCanvas.LEFT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}
	
	void edge5Pos0(int face, int position) {
		if (face == 4 && edgeToIndex(position) == 0) {
			simpleEdgeMove(2, OctCanvas.RIGHT, 1, OctCanvas.TOP);
		} else if (face == 4 && edgeToIndex(position) == 1) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(2, OctCanvas.RIGHT, 1, OctCanvas.TOP);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			simpleEdgeMove(1, OctCanvas.TOP, 2, OctCanvas.RIGHT);
		} else if (face == 6 && edgeToIndex(position) == 0) {
			simpleEdgeMove(2, OctCanvas.LEFT, 3, OctCanvas.TOP);
			simpleEdgeMove(1, OctCanvas.TOP, 2, OctCanvas.RIGHT);
		} else if (face == 6 && edgeToIndex(position) == 1) {
			return;
		}
	}
	
	void edge5Pos1(int face, int position) {
		if (face == 4 && edgeToIndex(position) == 0) {
			return;
		} else if (face == 4 && edgeToIndex(position) == 1) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleEdgeMove(2, OctCanvas.LEFT, 3, OctCanvas.TOP);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(3, OctCanvas.TOP, 2, OctCanvas.LEFT);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (face == 6 && edgeToIndex(position) == 0) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleEdgeMove(2, OctCanvas.LEFT, 3, OctCanvas.TOP);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		}
	}
	
	void edge2Pos2(int face, int position) {
		if (face == 4 && edgeToIndex(position) == 1) {
			simpleEdgeMove(3, OctCanvas.TOP, 2, OctCanvas.LEFT);
		} else if (face == 4 && edgeToIndex(position) == 2) {
			return;
		} else if (face == 6 && edgeToIndex(position) == 0) {
			simpleEdgeMove(2, OctCanvas.LEFT, 3, OctCanvas.TOP);
		}
	}
	
	void solveTopEdges() {
		int topColor = oct.facetLoc[0][0].face;
		int otherColor = oct.facetLoc[6][0].face;
		findPiece(otherColor, topColor);
		if (DEBUG)
			System.out.println("Top face " + returnedFace
				+ ", position " + returnedPosition);
		edge0Pos2(returnedFace, returnedPosition);
		otherColor = oct.facetLoc[1][0].face;
		findPiece(otherColor, topColor);
		if (DEBUG)
			System.out.println("Right face " + returnedFace
				+ ", position " + returnedPosition);
		edge0Pos0(returnedFace, returnedPosition);
		otherColor = oct.facetLoc[3][0].face;
		findPiece(otherColor, topColor);
		if (DEBUG)
			System.out.println("Left face " + returnedFace
				+ ", position " + returnedPosition);
		edge0Pos1(returnedFace, returnedPosition);
	}
	
	void solveMiddleEdges() {
		int sideColor = oct.facetLoc[1][0].face;
		int otherColor = oct.facetLoc[5][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Right face " + returnedFace
				+ ", position " + returnedPosition);
		edge1Pos2(returnedFace, returnedPosition);
		sideColor = oct.facetLoc[3][0].face;
		otherColor = oct.facetLoc[7][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Left face " + returnedFace
				+ ", position " + returnedPosition);
		edge3Pos2(returnedFace, returnedPosition);
		sideColor = oct.facetLoc[1][0].face;
		otherColor = oct.facetLoc[2][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Right face " + returnedFace
				+ ", position " + returnedPosition);
		edge1Pos0(returnedFace, returnedPosition);
		sideColor = oct.facetLoc[3][0].face;
		otherColor = oct.facetLoc[2][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Left face " + returnedFace
				+ ", position " + returnedPosition);
		edge3Pos1(returnedFace, returnedPosition);
	}
	
	void solveBottomEdges() {
		int sideColor = oct.facetLoc[5][0].face;
		int otherColor = oct.facetLoc[6][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Bottom face " + returnedFace
				+ ", position " + returnedPosition);
		edge5Pos0(returnedFace, returnedPosition);
		otherColor = oct.facetLoc[4][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Top face " + returnedFace
				+ ", position " + returnedPosition);
		edge5Pos1(returnedFace, returnedPosition);
		sideColor = oct.facetLoc[2][0].face;
		otherColor = oct.facetLoc[4][0].face;
		findPiece(otherColor, sideColor);
		if (DEBUG)
			System.out.println("Last face " + returnedFace
				+ ", position " + returnedPosition);
		edge2Pos2(returnedFace, returnedPosition);
	}

	int[] centerFaces = new int[3];
	int[] centerPositions = new int[3];
	int[] centerRotations = new int[3];

	void findCenterPositions(int color) {
		int face, position, positionIndex, i = 0;

		for (face = 0; face < OctCanvas.MAX_FACES; face++) {
			for (positionIndex = 0; positionIndex < 3; positionIndex++) {
				position = indexToCenter(positionIndex);
				if (oct.facetLoc[face][position].face == color) {
					centerFaces[i] = face;
					centerPositions[i] = position;
					i++;
					if (i >= 3)
						return;
				}
			}
		}
	}

	void findCenterPositionsRev(int color) {
		int face, position, positionIndex, i = 0;

		for (face = OctCanvas.MAX_FACES - 1; face >= 0; face--) {
			for (positionIndex = 0; positionIndex < 3; positionIndex++) {
				position = indexToCenter(positionIndex);
				if (oct.facetLoc[face][position].face == color) {
					centerFaces[i] = face;
					centerPositions[i] = position;
					i++;
					if (i >= 3)
						return;
				}
			}
		}
	}

	void simpleCenterSwap(int face0, int dir0,
			int face1, int dir1) {
		int facePosition = indexToEdge(2);
		int centerPosition = indexToEdge(0);
		int i;

		for (i = 0; i < 5; i++) {
			movePuzzlePiece(face0, facePosition,
				dir0, OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, centerPosition,
				dir1, OctInterface.PERIOD3, false);
			movePuzzlePiece(face0, facePosition,
				oppDir(dir0), OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, centerPosition,
				oppDir(dir1), OctInterface.PERIOD3, false);
		}
	}

	void solve2To0Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("20 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(4, OctCanvas.RIGHT, 1, OctCanvas.BOTTOM);
		/* (0, 1):= (2, 1), (2, 2):= (0, 1), (2, 1):= (2, 2) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
	}

	void solve5To0Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("50 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(1, OctCanvas.TOP, 3, OctCanvas.TOP);
		/* (0, 2):= (5, 1), (5, 0):= (0, 2), (5, 1):= (5, 0) */
		if (targetPosition == indexToCenter(1))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve7To0Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("70 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(3, OctCanvas.TOP, 1, OctCanvas.TOP);
		/* (0, 2):= (5, 1), (5, 0):= (0, 2), (5, 1):= (5, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve5To2Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("52 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(1, OctCanvas.BOTTOM, 3, OctCanvas.BOTTOM);
		/* (2, 1):= (5, 2), (5, 0):= (2, 1), (5, 2):= (5, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void solve7To2Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("72 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(3, OctCanvas.BOTTOM, 1, OctCanvas.BOTTOM);
		/* (2, 2):= (7, 1), (7, 0):= (2, 2), (7, 1):= (7, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void solve7To5Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("75 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		simpleCenterSwap(3, OctCanvas.BOTTOM, 0, OctCanvas.RIGHT);
		/* (5, 2):= (7, 2), (7, 1):= (5, 2), (7, 2):= (7, 1) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve4To6Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("46 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(2, OctCanvas.LEFT, 7, OctCanvas.TOP);
		/* (6, 1):= (4, 1), (4, 2):= (6, 1), (4, 1):= (4, 2) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
	}

	void solve3To6Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("36 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(7, OctCanvas.BOTTOM, 5, OctCanvas.BOTTOM);
		/* (6, 2):= (3, 1), (3, 0):= (6, 2), (3, 1):= (3, 0) */
		if (targetPosition == indexToCenter(1))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void solve1To6Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("16 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(5, OctCanvas.BOTTOM, 7, OctCanvas.BOTTOM);
		/* (6, 2):= (3, 1), (3, 0):= (6, 2), (3, 1):= (3, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void solve3To4Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("34 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(7, OctCanvas.TOP, 5, OctCanvas.TOP);
		/* (4, 1):= (3, 2), (3, 0):= (4, 1), (3, 2):= (3, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve1To4Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("14 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(5, OctCanvas.TOP, 7, OctCanvas.TOP);
		/* (4, 2):= (1, 1), (1, 0):= (4, 2), (1, 1):= (1, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(2))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve1To3Faces(int sourcePosition, int targetPosition) {
		if (DEBUG) {
			System.out.println("13 sp=" + sourcePosition
				+ ", tp=" + targetPosition);
		}
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		simpleCenterSwap(5, OctCanvas.TOP, 6, OctCanvas.LEFT);
		/* (3, 2):= (1, 2), (1, 1):= (3, 2), (1, 2):= (1, 1) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		if (sourcePosition == indexToCenter(0))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (sourcePosition == indexToCenter(1))
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void solve0Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 0;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositions(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 2) {
				sourceFace = 2;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve2To0Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 5) {
				sourceFace = 5;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve5To0Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 7) {
				sourceFace = 7;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve7To0Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void solve2Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 2;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositions(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 5) {
				sourceFace = 5;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve5To2Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 7) {
				sourceFace = 7;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve7To2Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void solve5Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 5;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositions(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 7) {
				sourceFace = 7;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve7To5Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void solve6Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 6;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositionsRev(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 4) {
				sourceFace = 4;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve4To6Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 3) {
				sourceFace = 3;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve3To6Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 1) {
				sourceFace = 1;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve1To6Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void solve4Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 4;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositionsRev(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 3) {
				sourceFace = 3;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve3To4Faces(sourcePosition, targetPosition);
				}
			} else if (centerFaces[count] == 1) {
				sourceFace = 1;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve1To4Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void solve3Faces() {
		int count;
		int sourcePosition, targetPosition;
		int sourceFace, targetFace = 3;
		int color = oct.facetLoc[targetFace][0].face;

		if (DEBUG) {
			System.out.println("targetFace " + targetFace);
		}
		for (count = 0; count < 3; count++) {
			int sourceCount = 0, targetCount = 0;

			findCenterPositionsRev(color);
			if (DEBUG) {
				System.out.println(sourceFace
					+ ": face0=" + centerFaces[0]
					+ ", position0=" + centerPositions[0] 
					+ "; face1=" + centerFaces[1]
					+ ", position1=" + centerPositions[1] 
					+ "; face2=" + centerFaces[2]
					+ ", position2=" + centerPositions[2]);
			}
			if (centerFaces[count] == 1) {
				sourceFace = 1;
				while (oct.facetLoc[targetFace][indexToCenter(targetCount)].face == color) {
					targetCount++;
					if (targetCount >= 3)
						return;
				}
				while (centerFaces[sourceCount] != sourceFace) {
					sourceCount++;
					if (sourceCount >= 3)
						return;
				}
				if (DEBUG) {
					System.out.println(sourceFace
						+ ": face0=" + centerFaces[0]
						+ ", position0=" + centerPositions[0] 
						+ "; face1=" + centerFaces[1]
						+ ", position1=" + centerPositions[1] 
						+ "; face2=" + centerFaces[2]
						+ ", position2=" + centerPositions[2]
						+ ", targetCount " + targetCount 
						+ ", sourceCount " + sourceCount); 
				}
				if (centerFaces[sourceCount] == sourceFace) {
					sourcePosition = centerPositions[sourceCount];
					targetPosition = indexToCenter(targetCount);
					solve1To3Faces(sourcePosition, targetPosition);
				}
			}
		}
	}

	void swapOut0To2(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(4, OctCanvas.LEFT, 1, OctCanvas.BOTTOM);
		/* (0, 1):= (2, 1), (2, 2):= (0, 1), (2, 1):= (2, 2) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
	}

	void swapOut2To5(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(1, OctCanvas.BOTTOM, 3, OctCanvas.BOTTOM);
		/* (2, 1):= (5, 2), (5, 0):= (2, 1), (5, 2):= (5, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
	}

	void swapOut5To7(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		simpleCenterSwap(3, OctCanvas.BOTTOM, 0, OctCanvas.RIGHT);
		/* (5, 2):= (7, 2), (7, 1):= (5, 2), (7, 2):= (7, 1) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
	}

	void swapOut6To4(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		simpleCenterSwap(2, OctCanvas.LEFT, 7, OctCanvas.TOP);
		/* (6, 1):= (4, 1), (4, 2):= (6, 1), (4, 1):= (4, 2) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
	}

	void swapOut4To3(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		simpleCenterSwap(7, OctCanvas.TOP, 5, OctCanvas.TOP);
		/* (4, 1):= (3, 2), (3, 0):= (4, 1), (3, 2):= (3, 0) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(2))
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
	}

	void swapOut3To1(int targetPosition) {
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		simpleCenterSwap(5, OctCanvas.TOP, 6, OctCanvas.LEFT);
		/* (3, 2):= (1, 2), (1, 1):= (3, 2), (1, 2):= (1, 1) */
		if (targetPosition == indexToCenter(0))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (targetPosition == indexToCenter(1))
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
	}

	void solve0Orient() {
		int count, found = 0;
		int targetFace = 0;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve0Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve0Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut0To2(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(1)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(4, OctCanvas.LEFT, 3, OctCanvas.BOTTOM);
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(4, OctCanvas.LEFT, 3, OctCanvas.BOTTOM);
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(4, OctCanvas.LEFT, 3, OctCanvas.BOTTOM);
		}
		if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(4, OctCanvas.RIGHT, 1, OctCanvas.BOTTOM);
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(4, OctCanvas.RIGHT, 1, OctCanvas.BOTTOM);
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(4, OctCanvas.RIGHT, 1, OctCanvas.BOTTOM);
		}
		if (found == 3) {
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(4, OctCanvas.LEFT, 3, OctCanvas.BOTTOM);
			movePuzzlePiece(6, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		}
	}

	void solve2Orient() {
		int count, found = 0;
		int targetFace = 2;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve2Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve2Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut2To5(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 2)) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.BOTTOM, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.BOTTOM, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.BOTTOM, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
		if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.TOP, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.TOP, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(1, OctCanvas.TOP, 5, OctCanvas.BOTTOM);
		}
		if (found == 3) {
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(1, OctCanvas.BOTTOM, 5, OctCanvas.BOTTOM);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		}
	}

	void solve5Orient() {
		int count, found = 0;
		int targetFace = 5;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve5Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve5Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut5To7(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(1)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
		if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.TOP, 6, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.TOP, 6, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.TOP, 6, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
		if (found == 3) {
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			simpleCenterSwap(3, OctCanvas.BOTTOM, 4, OctCanvas.LEFT);
			movePuzzlePiece(3, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}

	void solve7Orient() {
		int count, found = 0, i;
		int targetFace = 7;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			System.out.println("solve7Orient: found=3, centerRot=" +
				centerRotations[0]);
		}
		for (i = 0; i < centerRotations[0]; i++) {
			movePuzzlePiece(7, indexToEdge(0),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			swapOut5To7(centerPositions[0]);
			movePuzzlePiece(5, indexToEdge(0),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(4, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		}
	}

	void solve6Orient() {
		int count, found = 0;
		int targetFace = 6;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve6Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve6Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut6To4(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(1)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(2, OctCanvas.RIGHT, 5, OctCanvas.TOP);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(2, OctCanvas.RIGHT, 5, OctCanvas.TOP);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(2, OctCanvas.RIGHT, 5, OctCanvas.TOP);
		}
		if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(2, OctCanvas.LEFT, 7, OctCanvas.TOP);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(2, OctCanvas.LEFT, 7, OctCanvas.TOP);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(2, OctCanvas.LEFT, 7, OctCanvas.TOP);
		}
		if (found == 3) {
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(2, OctCanvas.RIGHT, 5, OctCanvas.TOP);
			movePuzzlePiece(0, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		}
	}

	void solve4Orient() {
		int count, found = 0;
		int targetFace = 4;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve4Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve4Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut4To3(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 2)) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.TOP, 3, OctCanvas.TOP);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.TOP, 3, OctCanvas.TOP);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.TOP, 3, OctCanvas.TOP);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
		if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.BOTTOM, 3, OctCanvas.TOP);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		} else if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.BOTTOM, 3, OctCanvas.TOP);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		} else {
			simpleCenterSwap(7, OctCanvas.BOTTOM, 3, OctCanvas.TOP);
		}
		if (found == 3) {
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(7, OctCanvas.TOP, 3, OctCanvas.TOP);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
		}
	}

	void solve3Orient() {
		int count, found = 0;
		int targetFace = 3;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			if (found == 2)
				System.out.println("solve1Orient: found=2, centerRot=" +
					centerPositions[0]);
			else
				System.out.println("solve1Orient: found=3, centerRot=" +
					centerRotations[0]);
		}
		swapOut3To1(centerPositions[0]);
		if ((found == 2 && centerPositions[1] == indexToCenter(1)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.TOP, 2, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[1] == indexToCenter(0)) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.TOP, 2, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.TOP, 2, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
		if ((found == 2 && centerPositions[0] == indexToCenter(2)) ||
				((found == 3) && centerRotations[0] == 1)) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.BOTTOM, 0, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		} else if (found == 2 && centerPositions[0] == indexToCenter(0)) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.BOTTOM, 0, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		} else {
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.BOTTOM, 0, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
		if (found == 3) {
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			simpleCenterSwap(5, OctCanvas.TOP, 2, OctCanvas.RIGHT);
			movePuzzlePiece(5, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			movePuzzlePiece(7, indexToEdge(2),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
	}

	void solve1Orient() {
		int count, found = 0, i;
		int targetFace = 1;
		int faceRotation = oct.facetLoc[targetFace][0].rotation;

		if (DEBUG)
			System.out.println("targetFace " + targetFace
				+ ": " + faceRotation);
		for (count = 0; count < 3; count++) {
			centerPositions[found] = indexToCenter(count);
			centerRotations[found] = ((oct.facetLoc[targetFace][centerPositions[found]].rotation - faceRotation + 12) % 12) / 4;
			if (centerRotations[found] != 0) {
				found++;
			}
		}
		if (found == 0)
			return;
		if (DEBUG) {
			for (count = 0; count < found; count++) {
				System.out.println(targetFace
					+ " count " + count
					+ ": centerPos " + centerPositions[count]
					+ ", centerRot " + centerRotations[count]); 
			}
			System.out.println("solve1Orient: found=3, centerRot=" +
				centerRotations[0]);
		}
		for (i = 0; i < centerRotations[0]; i++) {
			movePuzzlePiece(1, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
			movePuzzlePiece(3, indexToEdge(0),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
			swapOut3To1(centerPositions[0]);
			movePuzzlePiece(3, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
			movePuzzlePiece(2, indexToEdge(2),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
			movePuzzlePiece(1, indexToEdge(0),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		}
	}

	void solveCenterEarly() {
		int startFace = 0;
		int color = oct.facetLoc[startFace][0].face;
		int face = findPieceCenter(color, startFace);
		if (face == 2)
			movePuzzlePiece(3, indexToEdge(0),
				OctCanvas.TOP, OctInterface.PERIOD3, false);
		else if (face == 5)
			movePuzzlePiece(3, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		else if (face == 7)
			movePuzzlePiece(1, indexToEdge(0),
				OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
		startFace = 1;
		color = oct.facetLoc[startFace][0].face;
		face = findPieceCenter(color, startFace);
		if (face == 6)
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.LEFT, OctInterface.PERIOD3, false);
		else if (face == 3)
			movePuzzlePiece(2, indexToEdge(0),
				OctCanvas.RIGHT, OctInterface.PERIOD3, false);
	}

	void solveCenter() {
		int i;
		int face0 = 0;
		int color0 = oct.facetLoc[face0][0].face;
		int face01 = findPieceCenter(color0, face0);
		int color01 = oct.facetLoc[face01][0].face;
		int face02 = findPieceCenter(color01, face01);
		int face2 = 2;
		int color2 = oct.facetLoc[face2][0].face;
		int face21 = findPieceCenter(color2, face2);
		int color21 = oct.facetLoc[face21][0].face;
		int face22 = findPieceCenter(color21, face21);
		int face1 = 1;
		int color1 = oct.facetLoc[face1][0].face;
		int face11 = findPieceCenter(color1, face1);
		int face3 = 3;
		int color3 = oct.facetLoc[face3][0].face;
		int face31 = findPieceCenter(color3, face3);
		if (DEBUG) {
			System.out.println("solve color0 " + color0
				+ ", face01 " + face01
				+ ", color01 " + color01
				+ ", face02 " + face02);
			System.out.println("solve color2 " + color2
				+ ", face21 " + face01
				+ ", color21 " + color01
				+ ", face22 " + face22);
		}
		if (face0 == face01 && face0 == face02 && face2 == face22)
			return;
		else if (face0 == face02 && face2 == face22) {
			/* simple */
			if (face01 == 2) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
				}
			} else if (face01 == 5) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
			} else if (face01 == 7) {
				for (i = 0; i < 3; i++) {
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(1, indexToEdge(0),
						OctCanvas.TOP, OctInterface.PERIOD3, false);
					movePuzzlePiece(3, indexToEdge(0),
						OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
				}
			}
		} else {
			/* CMove */
			if (face0 == face01) {
				if (face11 == 3) {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(1, indexToEdge(0),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
						movePuzzlePiece(0, indexToEdge(0),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
						movePuzzlePiece(3, indexToEdge(0),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					}
				} else {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(3, indexToEdge(1),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
						movePuzzlePiece(0, indexToEdge(1),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
						movePuzzlePiece(1, indexToEdge(1),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
					}
				}
			} else if (face1 == face11) {
				if (face01 == 2) {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(0, indexToEdge(1),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
						movePuzzlePiece(1, indexToEdge(1),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
						movePuzzlePiece(2, indexToEdge(1),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
					}
				} else {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(2, indexToEdge(0),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
						movePuzzlePiece(1, indexToEdge(0),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
						movePuzzlePiece(0, indexToEdge(0),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
					}
				}
			} else if (face2 == face21) {
				if (face31 == 1) {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(3, indexToEdge(0),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
						movePuzzlePiece(2, indexToEdge(0),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
						movePuzzlePiece(1, indexToEdge(0),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
					}
				} else {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(1, indexToEdge(1),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
						movePuzzlePiece(2, indexToEdge(1),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
						movePuzzlePiece(3, indexToEdge(1),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
					}
				}
			} else if (face3 == face31) {
				if (face21 == 0) {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(2, indexToEdge(1),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
						movePuzzlePiece(3, indexToEdge(1),
							OctCanvas.TOP, OctInterface.PERIOD3, false);
						movePuzzlePiece(0, indexToEdge(1),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					}
				} else {
					for (i = 0; i < 8; i++) {
						movePuzzlePiece(0, indexToEdge(0),
							OctCanvas.LEFT, OctInterface.PERIOD3, false);
						movePuzzlePiece(3, indexToEdge(0),
							OctCanvas.BOTTOM, OctInterface.PERIOD3, false);
						movePuzzlePiece(2, indexToEdge(0),
							OctCanvas.RIGHT, OctInterface.PERIOD3, false);
					}
				}
			}
		}
	}

	void fixOrientCenterFace(Boolean opp,
			int face0, int dir0, int face1, int dir1) {
		int i, direction0, direction1;

		if (opp) {
			direction0 = dir0;
			direction1 = dir1;
		} else {
			direction0 = oppDir(dir0);
			direction1 = oppDir(dir1);
		}
		for (i = 0; i < 140; i++) {
			movePuzzlePiece(face0, indexToEdge(2),
				direction0, OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, indexToEdge(2),
				direction1, OctInterface.PERIOD3, false);
		}
	}

	int totalPieceCenterOrient() {
		int position = exactCenter();
		int face, rotation, totalRotation = 0;

		for (face = 0; face < OctCanvas.MAX_FACES; face++) {
			rotation = (oct.facetLoc[face][position].rotation
				- oct.facetLoc[face][0].rotation
				+ OctCanvas.COORD + 1) % 3;
			totalRotation += rotation;
		}
		return totalRotation % 3;
	}

	void solveCenterOrient() {
		int face = 0;
		int value = findPieceCenterOrient(face);
		Boolean opp;

		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 6, OctCanvas.RIGHT, 5, OctCanvas.BOTTOM);
		}
		face = 1;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 4, OctCanvas.RIGHT, 5, OctCanvas.TOP);
		}
		face = 3;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 4, OctCanvas.RIGHT, 7, OctCanvas.BOTTOM);
		}
		face = 2;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 2, OctCanvas.LEFT, 4, OctCanvas.LEFT);
		}
		face = 4;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 2, OctCanvas.RIGHT, 1, OctCanvas.BOTTOM);
		}
		face = 5;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 0, OctCanvas.RIGHT, 1, OctCanvas.TOP);
		}
		face = 7;
		value = findPieceCenterOrient(face);
		if (value != 0) {
			opp = (value == 1);
			if (DEBUG) {
				System.out.println("solveCenterOrient " + face + " " + value);
			}
			fixOrientCenterFace(opp, 0, OctCanvas.RIGHT, 3, OctCanvas.BOTTOM);
		}
	}


	void mixup(int rotation) {
		int i, dir0 = OctCanvas.LEFT, dir1 = OctCanvas.BOTTOM;
		int face0 = 6, face1 = 5;

		if (rotation == 1) {
			dir0 = OctCanvas.BOTTOM;
			dir1 = OctCanvas.LEFT;
			face0 = 5;
			face1 = 6;
		}
		for (i = 0; i < 10; i++) {
			movePuzzlePiece(face0, indexToEdge(2),
				dir0, OctInterface.PERIOD3, false);
			movePuzzlePiece(face1, indexToEdge(2),
				dir1, OctInterface.PERIOD3, false);
		}
	}

	void solveFaces() {
		int rotation;

		solveTopCorners();
		solveBottomCorners();
		if (oct.size % 3 != 0) {
			if (oct.orient && oct.size > 2 && oct.size % 2 == 0) {
				solveCenterEarly();
				rotation = totalPieceCenterOrient();
				if (DEBUG) {
					System.out.println("rotation " + rotation);
				}
				if (rotation != 0) {
					mixup(rotation);
					solveTopCorners();
					solveBottomCorners();
					solveCenterEarly();
				}
			}
			solveTopEvenBottomOddCenters();
			solveTopOddBottomEvenCenters();
			if (oct.orient) {
				rotateTopHalf();
				rotateBottomHalf();
			}
		}
		if (oct.size > 2) {
			if (oct.sticky || oct.size % 2 != 0) {
				solveTopEdges();
				solveMiddleEdges();
				solveBottomEdges();
			}
			if (oct.size > 2 && oct.size % 2 == 0) {
				solveCenter();
			}
			solve0Faces();
			solve2Faces();
			solve5Faces();
			solve6Faces();
			solve4Faces();
			solve3Faces();
			if (oct.orient) {
				rotation = totalPieceCenterOrient();
				solve0Orient();
				solve2Orient();
				solve5Orient();
				solve7Orient();
				solve6Orient();
				solve4Orient();
				solve3Orient();
				solve1Orient();
				if (oct.size > 2 && oct.size % 2 == 0) {
					if (DEBUG) {
						rotation = totalPieceCenterOrient();
						System.out.println("totalRotation " + rotation);
					}
					solveCenterOrient();
				}
			}
		}
	}

	/* This procedure coordinates the solution process. */
	void solvePieces() {
		((OctFrame) frame).callback(
			OctInterface.ACTION_RESET);
		if (!oct.checkSolved()) {
			if (oct.period == OctInterface.PERIOD3)
				solveFaces();
			else if (oct.period == OctInterface.PERIOD4)
				solveCorners();
		}
		((OctFrame) frame).callback(
			OctInterface.ACTION_COMPUTED);
	}

	private class SeparateSubTask extends Thread {
		private boolean runFlag = false;
		SeparateSubTask() {
		}
		public void init() {
			start();
		}
		public void setFlag(boolean flag) {
			runFlag = flag;
		}
		public boolean getFlag() {
			return runFlag;
		}

		@Override
		public void run() {
			while (true) {
				try {
					sleep(100);
				} catch (InterruptedException e) {
					System.err.println("Interrupted");
				}
				if (runFlag) {
					solvePieces();
					runFlag = false;
				}
			}
		}
	}

	private SeparateSubTask sp = null;
	public void setFlag(boolean flag) {
		if (sp != null)
			sp.setFlag(flag);
	}
	public boolean getFlag() {
		if (sp == null)
			return false;
		return sp.getFlag();
	}
	public void init() {
		if (sp == null) {
			sp = new SeparateSubTask();
			sp.init();
		}
	}
}
